
UNIT MOUSE;         (* Version 1.00 *)

(* GESTION DE LA SOURIS EN MODE TEXTE.
   UNITE DESTINEE A FOURNIR DES UTILITAIRES SOURIS A L'UNITE TXT. *)

INTERFACE

Uses DOS;

Function TEST_SOURIS (var Nombre_Boutons:byte) : boolean;
   (* TESTE LA PRESENCE D'UN DRIVER DE SOURIS *)
Procedure TYPE_SOURIS (var Version,Sous_Version,Sorte:byte);
   (* DETERMINE LE TYPE DE SOURIS ET LA VERSION DU DRIVER *)
Procedure PARAMETRES_SOURIS (var Boutons,x,y:word);
   (* DETERMINATION DE LA POSITION DU CURSEUR ET DES BOUTONS ENFONCES *)
Function TAILLE_BUFFER_SAUVEGARDE : word;
   (* DETERMINE LA TAILLE DU BUFFER A RESERVER POUR LA SAUVEGARDE *)
Function PAGE_SOURIS : word;
   (* DETERMINE LA PAGE D'ECRAN OU LE DRIVER TRAVAILLE *)
Procedure FIXER_PAGE_ECRAN (Page:word);
   (* FIXE LA PAGE D'ECRAN ACTIVE POUR LE DRIVER *)
Procedure SAUVEGARDER_RESTAURER_ETAT (Option:byte; Buffer:pointer);
   (* SAUVEGARDE/RESTAURATION DE L'ETAT DE LA SOURIS DANS UN BUFFER *)
Procedure AFFICHER_EFFACER_CURSEUR (Option:byte);
   (* AFFICHE/EFFACE LE CURSEUR DE LA SOURIS A L'ECRAN *)
Procedure ZONE_DEPLACEMENT (x1,y1,x2,y2:word);
   (* DEFINITION DE LA ZONE DE DEPLACEMENT DE LA SOURIS *)
Procedure ZONE_DISPARITION (x1,y1,x2,y2:word);
   (* DEFINITION DE LA ZONE DE DISPARITION DE LA SOURIS *)
Procedure DEFINITION_CURSEUR (Type_Curseur,Word1,Word2:word);
   (* DEFINITION DU CURSEUR DE LA SOURIS EN MODE TEXTE *)
Procedure FIXER_VITESSE_SOURIS (Horiz,Vert,Seuil_Doublement:word);
   (* FIXE LA VITESSE ET LE SEUIL DE DOUBLEMENT DE LA VITESSE DE LA SOURIS *)
Procedure VITESSE_SOURIS (var Horiz,Vert,Seuil_Doublement:word);
   (* DETERMINE LA VITESSE ET LE SEUIL DE DOUBLEMENT DE LA VITESSE DE LA SOURIS *)
Procedure DEPLACER_CURSEUR (x,y:word);
   (* DEPLACE LE CURSEUR DE LA SOURIS *)
Procedure MOUVEMENTS_CURSEUR (var x,y:integer);
   (* DETERMINE LA DISTANCE ENTRE LA POSITION ACTUELLE ET LE PRECEDENT APPEL DE LA PROCEDURE *)
Function INSTALLER_GESTIONNAIRE_EVENEMENTS (Handler_Type:byte; Masque:word; Adresse:longint) : boolean;
   (* INSTALLATION D'UN GESTIONNAIRE D'EVENEMENTS *)
Function DESACTIVER_DRIVER (var Segment,Offset:word) : boolean;
   (* DESACTIVE LE DRIVER DE LA SOURIS ET RETOURNE SON ADRESSE *)
Procedure REACTIVER_DRIVER;
   (* REACTIVE LE DRIVER DESACTIVE PAR desactiver_Driver *)
Procedure NOMBRE_FOIS_BOUTON (Option:byte; Bouton:word; var NbFois,x,y:word);
   (* DETERMINE LE NOMBRE DE FOIS QU'UN BOUTON A ETE ENFONCE OU RELACHE *)
Procedure FREQUENCE_INTERRUPTION (Code_Frequence:word);
   (* FIXE LA FREQUENCE D'INTERRUPTION DU DRIVER DE SOURIS *)
Function INFORMATIONS_DRIVER : word;
   (* RETOURNE DES INFORMATIONS SUR LE DRIVER ET LE CURSEUR *)
Procedure MASQUES_CURSEUR (var Mot1,Mot2:word);
   (* DETERMINE LES MASQUES DE BITS DU CURSEUR *)
Function COMPTEUR_CURSEUR : word;
   (* RETOURNE LE COMPTEUR DE CURSEUR INTERNE *)


IMPLEMENTATION

Function TEST_SOURIS (var Nombre_Boutons:byte) : boolean;
(* TESTE LA PRESENCE D'UN DRIVER DE SOURIS.
   SI CE DRIVER EXISTE, LA FONCTION RENVOIE EGALEMENT LE NOMBRE DE BOUTONS DE
   LA SOURIS.
   VALEURS RETOURNEES PAR LA FONCTION : True = DRIVER PRESENT,
                                        False = DRIVER ABSENT. *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ax:=$0000;
      intr($33,Registres);
      if ax=$FFFF
         then
           begin
             TEST_SOURIS:=true;
             Nombre_Boutons:=bl;
           end
         else
           TEST_SOURIS:=false;
    end;
End;


Procedure TYPE_SOURIS (var Version,Sous_Version,Sorte:byte);
(* DETERMINE LE TYPE DE SOURIS ET LA VERSION DU DRIVER *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ax:=$0024;
      intr($33,Registres);
      Version:=bh; Sous_Version:=bl;
      Sorte:=ch;
    end;
End;


Procedure PARAMETRES_SOURIS (var Boutons,x,y:word);
(* DETERMINATION DE LA POSITION DU CURSEUR ET DES BOUTONS ENFONCES *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ax:=$0003;
      intr($33,Registres);
      Boutons:=bx;
      x:=cx shr 3; y:=dx shr 3;
    end;
End;


Function TAILLE_BUFFER_SAUVEGARDE : word;
(* DETERMINE LA TAILLE DU BUFFER A RESERVER POUR LA SAUVEGARDE *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ax:=$0015;
      intr($33,Registres);
      TAILLE_BUFFER_SAUVEGARDE:=bx;
    end;
End;


Function PAGE_SOURIS : word;
(* DETERMINE LA PAGE D'ECRAN OU LE DRIVER TRAVAILLE *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ax:=$001E;
      intr($33,Registres);
      PAGE_SOURIS:=bx;
    end;
End;


Procedure FIXER_PAGE_ECRAN (Page:word);
(* FIXE LA PAGE D'ECRAN ACTIVE POUR LE DRIVER *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ax:=$001D;
      bx:=Page;
      intr($33,Registres);
    end;
End;


Procedure SAUVEGARDER_RESTAURER_ETAT (Option:byte; Buffer:pointer);
(* SAUVEGARDE (Option 0)/ RESTAURATION (Option 1) DE L'ETAT DE LA SOURIS DANS UN BUFFER *)
Var Registres:registers;
Begin
  with Registres do
    begin
      if Option = 0 then ax:=$0016 else ax:=$0017;
      es:=seg(Buffer^); dx:=ofs(Buffer^);
      intr($33,Registres);
    end;
End;


Procedure AFFICHER_EFFACER_CURSEUR (Option:byte);
(* AFFICHE (Option=1)/EFFACE (Option=0) LE CURSEUR DE LA SOURIS A L'ECRAN *)
Var Registres:registers;
Begin
  with Registres do
    begin
      if Option = 1 then ax:=$0001 else ax:=$0002;
      intr($33,Registres);
    end;
End;


Procedure ZONE_DEPLACEMENT (x1,y1,x2,y2:word);
(* DEFINITION DE LA ZONE DE DEPLACEMENT DE LA SOURIS.
   (x1,y1) ET (x2,y2) SONT LES COORDONNEES ABSOLUES DE LA FENETRE. *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ax:=$0007;
      cx:=x1 shl 3; dx:=x2 shl 3;
      intr($33,Registres);
      ax:=$0008;
      cx:=x2 shl 3; dx:=y2 shl 3;
      intr($33,Registres);
    end;
End;


Procedure ZONE_DISPARITION (x1,y1,x2,y2:word);
(* DEFINITION DE LA ZONE DE DISPARITION DE LA SOURIS.
   (x1,y1) ET (x2,y2) SONT LES COORDONNEES ABSOLUES DE LA FENETRE. *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ax:=$0010;
      cx:=x1 shl 3; dx:=y1 shl 3;
      si:=x2 shl 3; di:=y2 shl 3;
      intr($33,Registres);
    end;
End;


Procedure DEFINITION_CURSEUR (Type_Curseur,Word1,Word2:word);
(* DEFINITION DU CURSEUR DE LA SOURIS EN MODE TEXTE.
   Type_Curseur : SI 0 ALORS CURSEUR LOGICIEL ET Word1 = MASQUE "AND" ET
                             Word2 = MASQUE "XOR" (ch ET dh SONT APPLIQUES
                             A L'ATTRIBUT ET cl ET dl AU CARACTERE);
                  SI 1 ALORS CURSEUR CLIGNOTANT ET Word1 ET Word2 SONT LES
                             LIGNES DE DEBUT ET DE FIN DU CURSEUR. *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ax:=$000A;
      bx:=Type_Curseur; cx:=Word1; dx:=Word2;
      intr($33,Registres);
    end;
End;


Procedure FIXER_VITESSE_SOURIS (Horiz,Vert,Seuil_Doublement:word);
(* DEFINITION DE LA VITESSE ET DU SEUIL DE DOUBLEMENT DE LA VITESSE DE LA
   SOURIS (EN MICKEYS/8 POINTS). *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ax:=$001A;
      bx:=Horiz; cx:=Vert; dx:=Seuil_Doublement;
      intr($33,Registres);
    end;
End;


Procedure VITESSE_SOURIS (var Horiz,Vert,Seuil_Doublement:word);
(* DETERMINE LA VITESSE ET LE SEUIL DE DOUBLEMENT DE LA VITESSE DE LA
   SOURIS (EN MICKEYS/8 POINTS). *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ax:=$001B;
      intr($33,Registres);
      Horiz:=bx; Vert:=cx; Seuil_Doublement:=dx;
    end;
End;


Procedure DEPLACER_CURSEUR (x,y:word);
(* DEPLACE LE CURSEUR DE LA SOURIS EN (x,y) (COORDONNEES CARACTERES) *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ax:=$0004;
      cx:=x shl 3; dx:=y shl 3;
      intr($33,Registres);
    end;
End;


Procedure MOUVEMENTS_CURSEUR (var x,y:integer);
(* DETERMINE LA DISTANCE ENTRE LA POSITION ACTUELLE ET LE PRECEDENT APPEL DE LA
   PROCEDURE, EN MICKEYS. *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ax:=$000B;
      intr($33,Registres);
      x:=cx; y:=dx;
    end;
End;


Function INSTALLER_GESTIONNAIRE_EVENEMENTS (Handler_Type:byte; Masque:word; Adresse:longint) : boolean;
(* INSTALLATION D'UN GESTIONNAIRE D'EVENEMENTS.
   Masque CONTIENT LES EVENEMENTS POUR LEQUEL LE GESTIONNAIRE DOIT ETRE APPELE
   PAR LE DRIVER DE LA SOURIS.
   SI Handler_Type VAUT 0, C'EST LE GESTIONNAIRE SIMPLE QUI EST CHARGE, TANDIS
   QUE SI Handler_Type VAUT 1, C'EST LE GESTIONNAIRE ALTERNATIF.
   VALEUR RETOURNEE PAR LA FONCTION : True = PAS D'ERREUR,
                                      False = ERREUR. *)
Type PtrRec=record
              Offset,Segment:word;
            end;
Var Registres:registers;
Begin
  with Registres do
    begin
      case Handler_Type of
        0:ax:=$000C;
        1:ax:=$0018;
      end;
      cx:=Masque;
      es:=PtrRec(Adresse).Segment; dx:=PtrRec(Adresse).Offset;
      intr($33,Registres);
      case Handler_Type of
        0:INSTALLER_GESTIONNAIRE_EVENEMENTS:=true;
        1:INSTALLER_GESTIONNAIRE_EVENEMENTS:=(ax = $0018);
      end;
    end;
End;


Function DESACTIVER_DRIVER (var Segment,Offset:word) : boolean;
(* DESACTIVE LE DRIVER DE LA SOURIS ET RETOURNE SON ADRESSE.
   VALEURS RETOURNEES PAR LA FONCTION : True = PAS D'ERREUR,
                                        False = ERREUR. *)
Var Registres:registers;
    Ok:boolean;
Begin
  with Registres do
    begin
      ax:=$001F;
      intr($33,Registres);
      Ok:=(ax = $001F);
      if Ok
         then
           begin
             Segment:=es; Offset:=bx;
           end;
    end;
  DESACTIVER_DRIVER:=Ok;
End;


Procedure REACTIVER_DRIVER;
(* REACTIVE LE DRIVER DESACTIVE PAR desactiver_Driver *)
Var Registres:registers;
Begin
  Registres.ax:=$0020;
  intr($33,Registres);
End;


Procedure NOMBRE_FOIS_BOUTON (Option:byte; Bouton:word; var NbFois,x,y:word);
(* DETERMINE LE NOMBRE DE FOIS QU'UN BOUTON (Bouton) A ETE ENFONCE (Option 0)
   OU RELACHE (Option 1) DEPUIS LE DERNIER APPEL A CETTE PROCEDURE.
   x,y SONT LES COORDONNEES DU CURSEUR LORS DE LA DERNIERE ACTION. *)
Var Registres:registers;
Begin
  with Registres do
    begin
      case option of
        0:ax:=$0005;
        1:ax:=$0006;
      end;
      bx:=Bouton;
      intr($33,Registres);
      NbFois:=bx;
      x:=cx shr 3; y:=dx shr 3;
    end;
End;


Procedure FREQUENCE_INTERRUPTION (Code_Frequence:word);
(* FIXE LA FREQUENCE D'INTERRUPTION DU DRIVER DE SOURIS.
   UNIQUEMENT SOURIS In Port. *)
Var registres:registers;
Begin
  with registres do
    begin
      ax:=$001C;
      bx:=Code_Frequence;
      intr($33,Registres);
    end;
End;


Function INFORMATIONS_DRIVER : word;
(* RETOURNE UN WORD AVEC DES INFORMATIONS SUR LE DRIVER ET LE CURSEUR.
   UNIQUEMENT VERSIONS SUPERIEURES A 6.26 *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ax:=$0025;
      intr($33,Registres);
      INFORMATIONS_DRIVER:=ax;
    end;
End;


Procedure MASQUES_CURSEUR (var Mot1,Mot2:word);
(* DETERMINE LES MASQUES DE BITS DU CURSEUR.
   SI CURSEUR ELECTRONIQUE, IL S'AGIT DES LIGNES DE DEBUT ET DE FIN;
   SI CURSEUR LOGICIEL, IL S'AGIT DES MASQUES and ET xor.
   UNIQUEMENT VERSIONS SUPERIEURES A 7.01 *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ax:=$0027;
      intr($33,Registres);
      Mot1:=ax; Mot2:=bx;
    end;
End;


Function COMPTEUR_CURSEUR : word;
(* RETOURNE LE COMPTEUR DE CURSEUR INTERNE.
   UNIQUEMENT VERSIONS SUPERIEURES A 7.02 *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ax:=$002A;
      intr($33,Registres);
      COMPTEUR_CURSEUR:=ax;
    end;
End;


END.
