
UNIT TESTADAP;

(* TESTS DE PRESENCE DE DIFFERENTS SYSTEMES VIDEO.

   IMPORTANT : LES TESTS DOIVENT ETRE FAITS DANS L'ORDRE SUIVANT :
   VGA/MCGA, EGA, CGA, MDA_OU_HERCULES.

   2 ADAPTATEURS SIMULTANES PEUVENT ETRE INSTALLES :

            MDA     Herc.   CGA     EGA m.  EGA c.  MCGA    VGA
   
    MDA                       XX              XX      XX      XX
    Herc.                     XX              XX      XX      XX
    CGA               XX              XX                   
    EGA m.                    XX                           
    EGA c.    XX      XX                                   
    MCGA      XX      XX                                   
    VGA       XX      XX                                   

   SEGMENTS DE RAM VIDEO : B000:0000..B000:7FFF = MDA, HERCULES, EGA MONO;
                           B800:0000..B800:7FFF = CGA, EGA, MCGA, VGA. *)


INTERFACE

Uses DOS;

Const Aucun = $00; Inconnu = $00; Erreur = $80;
      (* CARTES *)       (* MONITEURS *)
      Mda = $01;         Mono = $01;
      Cga = $02;         Couleur = $02;
      Hercules = $04;    Mono_Analogique = $04;
      Ega = $08;         Couleur_Analogique = $08;
      Mcga = $10;        Ega_HiRes = 10;
      Vga = $20;
      Svga_Vesa = $40;

Function TEST_VGA_MCGA (var Carte1,Moniteur1,Carte2,Moniteur2:byte) : boolean;
   (* TESTE LA PRESENCE D'UNE CARTE VGA OU MCGA *)
Function TEST_EGA (var Carte,Moniteur:byte) : boolean;
   (* TESTE LA PRESENCE D'UNE CARTE EGA *)
Function TEST_CGA (var Carte,Moniteur:byte) : boolean;
   (* TESTE LA PRESENCE D'UNE CARTE CGA *)
Function TEST_MDA_HERCULES (var Carte,Moniteur:byte) : boolean;
   (* TESTE LA PRESENCE D'UNE CARTE MONOCHROME *)
Function GETMODE (var Mode:byte) : boolean;
   (* DETERMINE LE MODE VIDEO COURANT *)


IMPLEMENTATION

Function TEST_MDA_OU_HERCULES : byte;
(* TESTE SI LA CARTE MONOCHROME EST UNE MDA OU UNE HERCULES.
   PRINCIPE : LE CONTROLEUR DE LA CARTE MONOCHROME (CRTC 6845) EST ACCESSIBLE
              PAR LE PORT 3B4h. POUR UNE CARTE MDA, LE BIT 7 DU REGISTRE RESTE
              TOUJOURS A LA MEME VALEUR, TANDIS QUE CETTE VALEUR OSCILLE SANS
              CESSE ENTRE 0 ET 1 POUR LA CARTE HERCULES.
   VALEURS RETOURNEES PAR LA FONCTION : Mda OU Hercules. *)
Var Valeur_Depart,Valeur_Courante:byte;
    Indice:integer;
Begin
  Valeur_Depart:=port[$03B4] and $80;   (* SEUL LE BIT 7 EST GARDE *)
  Indice:=0;
  repeat
    Valeur_Courante:=port[$03B4] and $80;
    inc(Indice);
  until (Indice = maxint) or (Valeur_Courante <> Valeur_Depart);
  if Indice = maxint then TEST_MDA_OU_HERCULES:=Mda else TEST_MDA_OU_HERCULES:=Hercules;
End;


Function TEST_VGA_OU_SVGA : byte;
(* TESTE SI LA CARTE INSTALLEE EST SVGA COMPATIBLE VESA OU VGA SIMPLE.
   PRINCIPE : LA FONCTION 4Fh DE L'INTERRUPTION 10h GERE LES FONCTIONNALITES
              VESA. LA SOUS-FONCTION 00h PERMET DE DETERMINER SI LES FONCTIONS
              VESA SONT SUPPORTEES.
   VALEURS RETOURNEES PAR LA FONCTION : Vga OU Svga_Vesa. *)
Var Registres:registers;
    Buffer:array[0..255] of byte;
Begin
  with Registres do
    begin
      ah:=$4F;
      al:=$00;
      es:=seg(Buffer); di:=ofs(Buffer);
      intr($10,Registres);
      if ah = 0 then TEST_VGA_OU_SVGA:=Svga_Vesa else TEST_VGA_OU_SVGA:=Vga;
    end;
End;


Function TEST_VGA_MCGA (var Carte1,Moniteur1,Carte2,Moniteur2:byte) : boolean;
(* TESTE SI UNE CARTE VGA OU MCGA EST INSTALLEE.
   PRINCIPE : APPELLE LA FONCTION 1Ah DE L'INTERRUPTION 10h.
              SI UNE CARTE VGA EST INSTALLEE, AL CONTIENT 1Ah EN SORTIE ET
              BL ET BH CONTIENNENT RESPECTIVEMENT LE SYSTEME ACTIF ET LE
              SYSTEME INACTIF INSTALLE.
   VALEURS RETOURNEES PAR LA FONCTION : True = PAS D'ERREUR,
                                        False = ERREUR. *)
Var Ok:boolean;
    Registres:registers;
Begin
  Ok:=true;
  with Registres do
    begin
      ah:=$1A;
      al:=$00;
      intr($10,Registres);
      if al=$1A
         then
           begin   (* IL Y A UNE CARTE VGA OU MCGA *)
             case bl of
               $00,$FF:begin Carte1:=Aucun; Moniteur1:=Aucun; end;
               $01:begin Carte1:=TEST_MDA_OU_HERCULES; Moniteur1:=Mono; end;
               $02:begin Carte1:=Cga; Moniteur1:=Couleur; end;
               $04:begin Carte1:=Ega; Moniteur1:=Ega_HiRes; end;
               $05:begin Carte1:=Ega; Moniteur1:=Mono; end;
               $07:begin Carte1:=TEST_VGA_OU_SVGA; Moniteur1:=Mono_Analogique; end;
               $08:begin Carte1:=TEST_VGA_OU_SVGA; Moniteur1:=Couleur_Analogique; end;
               $0A:begin Carte1:=Mcga; Moniteur1:=Couleur; end;
               $0B:begin Carte1:=Mcga; Moniteur1:=Mono_Analogique; end;
               $0C:begin Carte1:=Mcga; Moniteur1:=Couleur_Analogique; end;
             else
               Carte1:=Erreur; Moniteur1:=Erreur; Ok:=false;
             end;
             if Ok
                then
                  begin
                    case bh of
                      $00,$FF:begin Carte2:=Aucun; Moniteur2:=Aucun; end;
                      $01:begin Carte2:=TEST_MDA_OU_HERCULES; Moniteur2:=Mono; end;
                      $02:begin Carte2:=Cga; Moniteur2:=Couleur; end;
                      $04:begin Carte2:=Ega; Moniteur2:=Ega_HiRes; end;
                      $05:begin Carte2:=Ega; Moniteur2:=Mono; end;
                      $07:begin Carte2:=TEST_VGA_OU_SVGA; Moniteur2:=Mono_Analogique; end;
                      $08:begin Carte2:=TEST_VGA_OU_SVGA; Moniteur2:=Couleur_Analogique; end;
                      $0A:begin Carte2:=Mcga; Moniteur2:=Couleur; end;
                      $0B:begin Carte2:=Mcga; Moniteur2:=Mono_Analogique; end;
                      $0C:begin Carte2:=Mcga; Moniteur2:=Couleur_Analogique; end;
                    else
                      Carte2:=Erreur; Moniteur2:=Erreur; Ok:=false;
                    end;
                  end;
           end;
    end;
  TEST_VGA_MCGA:=Ok;
End;


Function TEST_EGA (var Carte,Moniteur:byte) : boolean;
(* TESTE LA PRESENCE D'UNE CARTE EGA.
   PRINCIPE : LA SOUS-FONCTION 10h DE LA FONCTION 12h DE L'INTERRUPTION 10h
              EST SUPPORTEE UNIQUEMENT PAR LE BIOS EGA. SI, APRES L'APPEL,
              LA VALEUR DE BL (10h) EST CHANGEE, IL S'AGIT D'UNE CARTE EGA.
              POUR LE TYPE DE MONITEUR, L'ANALYSE DE CL, QUI CONTIENT LES DIP'S
              DE LA CARTE, L'INDIQUERA.
   PARAMETRES RENVOYES : SI CARTE EGA, Carte CONTIENDRA Ega ET Moniteur LE TYPE
                         DE MONITEUR; SINON, Carte CONTIENT UNE AUTRE VALEUR ET
                         Moniteur EST INDETERMINE.
   VALEUR RENVOYEE PAR LA FONCTION : TOUJOURS True = PAS D'ERREUR *)
Var Registres:registers;
Begin
  with Registres do
    begin
      ah:=$12;
      bl:=$10;
      intr($10,registres);
      if bl <> $10
         then
           begin   (* CARTE EGA CONNECTEE *)
             Carte:=Ega;
             case (cl shr 1) of
               0,3:Moniteur:=Couleur;
               1,4:Moniteur:=Ega_HiRes;
               2,5:Moniteur:=Mono;
             end;
           end
         else
           begin   (* PAS DE CARTE VGA CONNECTEE *)
             Carte:=Aucun; Moniteur:=Aucun;
           end;
    end;
  TEST_EGA:=true;
End;


Function TEST_CGA (var Carte,Moniteur:byte) : boolean;
(* TESTE LA PRESENCE D'UNE CARTE CGA.
   PRINCIPE : IL S'AGIT DE TESTER LA PRESENCE DU CONTROLEUR 6845 A L'ADRESSE
              DE PORT 3D4h (CAR, SI CARTE MONOCHROME, LE 6845 SE TROUVE AU
              PORT 3B4h). POUR CELA, ON LIT LE REGISTRE 10 DU CONTROLEUR, ON
              Y CHARGE UNE NOUVELLE VALEUR; SI, APRES UNE COURTE PAUSE, LA
              VALEUR EST RESTEE INCHANGEE, C'EST QUE LE 6845 SE TROUVE BIEN LA.
   PARAMETRES RENVOYES : SI CARTE CGA, Carte CONTIENDRA Cga ET Moniteur
                         "Couleur"; SINON, Carte CONTIENT UNE AUTRE VALEUR ET
                         Moniteur EST INDETERMINE.
   VALEUR RENVOYEE PAR LA FONCTION : TOUJOURS True = PAS D'ERREUR *)
Var Valeur_Depart,Valeur_Relue:byte;
    Attente:byte;
Begin
  port[$03D4]:=$0A;   (* APPEL DU REGISTRE 10 DANS LE REGISTRE D'ADRESSE DU 6845 *)
  Valeur_Depart:=port[$03D5];   (* LECTURE DU REGISTRE DE DONNEES DU 6845 *)
  port[$03D5]:=$4F;   (* ECRITURE D'UNE VALEUR *)
  Attente:=0;
  repeat inc(Attente) until Attente=100;   (* PAUSE *)
  Valeur_Relue:=port[$03D5];   (* RELECTURE *)
  port[$03D5]:=Valeur_Depart;   (* RESTAURATION DE L'ANCIENNE VALEUR *)
  if Valeur_Depart = Valeur_Relue
     then
       begin   (* LE CONTROLEUR EST PRESENT : CARTE CGA *)
         Carte:=Cga; Moniteur:=Couleur;
       end
     else
       begin   (* PAS DE CARTE CGA CONNECTEE *)
         Carte:=Aucun; Moniteur:=Aucun;
       end;
  TEST_CGA:=true;
End;


Function TEST_MDA_HERCULES (var Carte,Moniteur:byte) : boolean;
(* TESTE LA PRESENCE D'UNE CARTE MONOCHROME.
   PRINCIPE : IL S'AGIT DE TESTER LA PRESENCE DU CONTROLEUR 6845 A L'ADRESSE
              DE PORT 3B4h (CAR, SI CARTE CGA, LE 6845 SE TROUVE AU
              PORT 3D4h). POUR CELA, ON LIT LE REGISTRE 10 DU CONTROLEUR, ON
              Y CHARGE UNE NOUVELLE VALEUR; SI, APRES UNE COURTE PAUSE, LA
              VALEUR EST RESTEE INCHANGEE, C'EST QUE LE 6845 SE TROUVE BIEN LA.
              ENSUITE, L'APPEL A LA FONCTION Test_Mda_Ou_Hercules PERMET DE
              DIFFERENCIER LES CARTES MDA ET HERCULES.
   PARAMETRES RENVOYES : SI CARTE MONOCHROME, Carte CONTIENDRA Mda OU Hercules
                         ET Moniteur "Mono"; SINON, Carte CONTIENT UNE AUTRE
                         VALEUR ET Moniteur EST INDETERMINE.
   VALEUR RENVOYEE PAR LA FONCTION : TOUJOURS True = PAS D'ERREUR *)
Var Valeur_Depart,Valeur_Relue:byte;
    Attente:byte;
Begin
  port[$03B4]:=$0A;   (* APPEL DU REGISTRE 10 DANS LE REGISTRE D'ADRESSE DU 6845 *)
  Valeur_Depart:=port[$03B5];   (* LECTURE DU REGISTRE DE DONNEES DU 6845 *)
  port[$03B5]:=$4F;   (* ECRITURE D'UNE VALEUR *)
  Attente:=0;
  repeat inc(Attente) until Attente=100;   (* PAUSE *)
  Valeur_Relue:=port[$03B5];   (* RELECTURE *)
  port[$03B5]:=Valeur_Depart;   (* RESTAURATION DE L'ANCIENNE VALEUR *)
  if Valeur_Depart = Valeur_Relue
     then
       begin   (* LE CONTROLEUR EST PRESENT : CARTE MONOCHROME *)
         Carte:=TEST_MDA_OU_HERCULES; Moniteur:=Couleur;
       end
     else
       begin   (* PAS DE CARTE CGA CONNECTEE *)
         Carte:=Aucun; Moniteur:=Aucun;
       end;
  TEST_MDA_HERCULES:=true;
End;


Function GETMODE (var Mode:byte) : boolean;
(* DETERMINE LE MODE VIDEO COURANT.
   RETOURNE TOUJOURS True = PAS D'ERREUR. *)
Var registres:registers;
Begin
  with registres do
    begin
      ah:=$0F;
      intr($10,registres);
      Mode:=al;
    end;
  GETMODE:=true;
End;

END.
