
UNIT TTO_FILE;

(* GESTION DES FICHIERS SUR DISQUE POUR LE PROGRAMME TTO 1.20 *)

INTERFACE

{$I-}   (* GESTION PERSONNELLE DES ERREURS D'ENTREES/SORTIES *)

Uses DOS,        (* FONCTIONS DISQUE DU DOS *)
     TXT,        (* GESTION DE L'ECRAN EN MODE TEXTE (VERSION 2.00) *)
     TTO_DECL,   (* DECLARATION DE CONSTANTES, TYPES ET VARIABLES GLOBALES *)
     TTO_HARD,   (* ECONOMISEUR D'ECRAN *)
     TTO_COMN,   (* MESSAGES D'ERREUR ET EXPLICATIONS AU BAS DE L'ECRAN *)
     TTO_LIST;   (* TRAITEMENT DES LISTES CHAINEES DE POINTEURS (SAUVETAGE/RESTAURATION) *)


Function CREATION_FICHIER (TypeFichier : byte) : boolean;
   (* CREATION D'UN FICHIER *)
Function OUVERTURE_FICHIER (TypeFichier : byte) : boolean;
   (* OUVERTURE D'UN FICHIER *)
Function AJOUT_FICHIER (TypeFichier : byte) : boolean;
   (* OUVERTURE D'UN FICHIER TEXTE EN MODE AJOUT *)
Function POSITION_FICHIER (TypeFichier : byte; var Position : longint) : boolean;
   (* LECTURE DE LA POSITION COURANTE DANS UN FICHIER *)
Function DEPLACEMENT_FICHIER (TypeFichier : byte; Position : longint) : boolean;
   (* DEPLACEMENT DE LA POSITION COURANTE DANS UN FICHIER *)
Function LECTURE_FICHIER (TypeFichier : byte) : boolean;
   (* LECTURE D'UN FICHIER *)
Function ECRITURE_FICHIER (TypeFichier : byte) : boolean;
   (* ECRITURE D'UN FICHIER *)
Function FERMETURE_FICHIER (TypeFichier : byte) : boolean;
   (* FERMETURE D'UN FICHIER *)
Function FIN_DE_FICHIER (TypeFichier : byte) : boolean;
   (* DETECTION DE FIN DE FICHIER (eof) *)

Procedure LECTURE_FICHIER_INI;
   (* LECTURE DES OPTIONS DANS LE FICHIER TTO.INI ET CHARGEMENT DES OPTIONS PAR DEFAUT DU PROGRAMME *)
Procedure ECRITURE_FICHIER_INI;
   (* EN FIN DE PROGRAMME, ECRITURE DES OPTIONS DANS TTO.INI ET BACKUP TTO.INI --> TTO.I~I *)
Function SAUVETAGE_SITUATION (Option : byte) : boolean;
   (* SAUVETAGE DE LA SITUATION SUR DISQUE *)
Function RECUPERATION_SITUATION : boolean;
   (* RECUPERATION DE LA SITUATION SAUVEE SUR DISQUE *)

Function CREATION_FICHIER_INDEX (TypeFichier, Numero : byte) : boolean;
   (* CREATION D'UN FICHIER D'INDEX POUR L'AFFICHAGE D'UN FICHIER TEXTE *)
Function CHARGEMENT_INDEX (TypeFichier : byte) : boolean;
   (* LECTURE D'UN FICHIER INDEX ET CHARGEMENT DU TABLEAU Texte *)
Function TEST_DATE_HEURE_FICHIERS : byte;
   (* TESTE SI LES DATE ET HEURE DES FICHIERS TTO.HLP ET TTOHLP.NDX SONT IDENTIQUES *)
Function ALIGNER_DATE_HEURE_FICHIERS : boolean;
   (* ATTRIBUE AU FICHIER TTOHLP.NDX LES MEMES DATE ET HEURE QUE TTO.HLP *)


IMPLEMENTATION

Function CREATION_FICHIER (TypeFichier : byte) : boolean;
(* CREATION D'UN FICHIER.
   RETOURNE true SI OK OU false SI ERREUR. *)
Var Ok,Avorter:boolean;
    Erreur:integer;
Begin
  Ok:=false; Avorter:=false;
  repeat
    case TypeFichier of
      tf_Categories:rewrite(FichierCategories);
      tf_Joueurs:rewrite(FichierJoueurs);
      tf_Matches:rewrite(FichierMatches);
      tf_Ini:rewrite(FichierIni);
      tf_Print:rewrite(FichierPrint);
      tf_Index:rewrite(FichierIndex);
    end;
    Erreur:=ioresult;
    if Erreur = 0 then Ok:=true else if TypeFichier = tf_Ini then Avorter:=true else Avorter:=not MESSAGE_ERREUR(Erreur);
  until (Ok) or (Avorter);
  CREATION_FICHIER:=Ok;
End;


Function OUVERTURE_FICHIER (TypeFichier : byte) : boolean;
(* OUVERTURE D'UN FICHIER.
   RETOURNE true SI OK OU false SI ERREUR. *)
Var Ok,Avorter:boolean;
    Erreur:integer;
Begin
  Ok:=false; Avorter:=false;
  repeat
    case TypeFichier of
      tf_Categories:reset(FichierCategories);
      tf_Joueurs:reset(FichierJoueurs);
      tf_Matches:reset(FichierMatches);
      tf_Ini:reset(FichierIni);
      tf_Aide:reset(FichierAide);
      tf_Print:reset(FichierPrint);
      tf_Index:reset(FichierIndex);
    end;
    Erreur:=ioresult;
    if Erreur = 0
       then
         Ok:=true
       else
         if TypeFichier in [tf_Ini,tf_Index] then Avorter:=true else Avorter:=not MESSAGE_ERREUR(Erreur);
  until (Ok) or (Avorter);
  OUVERTURE_FICHIER:=Ok;
End;


Function AJOUT_FICHIER (TypeFichier : byte) : boolean;
(* OUVERTURE D'UN FICHIER TEXTE EN MODE AJOUT (APPEND).
   RETOURNE true SI OK OU false SI ERREUR. *)
Var Ok,Avorter:boolean;
    Erreur:integer;
Begin
  Ok:=false; Avorter:=false;
  repeat
    case TypeFichier of
      tf_Print:append(FichierPrint);
    end;
    Erreur:=ioresult;
    if Erreur = 0 then Ok:=true else Avorter:=not MESSAGE_ERREUR(Erreur);
  until (Ok) or (Avorter);
  AJOUT_FICHIER:=Ok;
End;


Function TEXTFILEPOS (var Fichier : text) : longint;
(* DETERMINATION DU POINTEUR COURANT D'UN FICHIER TEXTE.
   DETOURNEMENT DE LA FONCTION ORIGINALE filepos, EN FAISANT CROIRE AU COMPILATEUR
   QU'IL NE S'AGIT PAS D'UN FICHIER TEXTE.
   UNE VARIABLE DE TYPE FICHIER POINTE SUR UN RECORD DE TYPE filerec :
      filerec = record
                  handle, mode, recsize : word;
                  private : array [1..26] of byte;
                  userdata : array [1..16] of byte;
                  name : array [0..79] of char;
                end;
   UNE VARIABLE DE TYPE FICHIER TEXTE POINTE SUR UN RECORD DE TYPE textrec :
      textrec = record
                  handle, mode, bufsize, private, bufpos, bufend : word;
                  bufptr : ^charbuf;
                  openfunc, inoutfunc, flushfunc, closefunc : pointer;
                  userdata : array [1..16] of byte;
                  name : array [0..79] of char;
                  buffer : charbuf;
                end;
   COMME C'EST LE filepos ORIGINAL QUI EST UTILISE, ioresult AURA UNE VALEUR SIGNIFICATIVE. *)
Var Adresse:pointer;
    Position:longint;
Begin
  Adresse:=@Fichier;
  filerec(Adresse^).mode:=fminout; filerec(Adresse^).recsize:=1;
  textrec(Fichier).mode:=fminput; textrec(Fichier).bufsize:=sizeof(textbuf);
  filerec(Adresse^).mode:=fminout; filerec(Adresse^).recsize:=1;
  Position:=filepos(file(Adresse^)); TEXTFILEPOS:=Position - textrec(Fichier).bufend + textrec(Fichier).bufpos;
  textrec(Fichier).mode:=fminput; textrec(Fichier).bufsize:=sizeof(textbuf);
End;


Procedure TEXTSEEK (var Fichier : text; Deplacement : longint);
(* DEPLACEMENT DU POINTEUR D'UN FICHIER TEXTE.
   DETOURNEMENT DE LA FONCTION ORIGINALE seek, EN FAISANT CROIRE AU COMPILATEUR
   QU'IL NE S'AGIT PAS D'UN FICHIER TEXTE.
   COMME C'EST LE seek ORIGINAL QUI EST UTILISE, ioresult AURA UNE VALEUR SIGNIFICATIVE. *)
Var Adresse:pointer;
Begin
  Adresse:=@Fichier;
  filerec(Adresse^).mode:=fminout; filerec(Adresse^).recsize:=1;
  textrec(Fichier).mode:=fminput; textrec(Fichier).bufsize:=sizeof(textbuf);
  filerec(Adresse^).mode:=fminout; filerec(Adresse^).recsize:=1;
  seek(file(Adresse^),Deplacement);
  textrec(Fichier).mode:=fminput; textrec(Fichier).bufsize:=sizeof(textbuf);
  textrec(Fichier).bufpos:=0; textrec(Fichier).bufend:=0;
End;


Function POSITION_FICHIER (TypeFichier : byte; var Position : longint) : boolean;
(* LECTURE DE LA POSITION COURANTE DANS UN FICHIER (filepos).
   RETOURNE true SI OK OU false SI ERREUR. *)
Var Ok,Avorter:boolean;
    Erreur:integer;
Begin
  Ok:=false; Avorter:=false;
  repeat
    case TypeFichier of
      tf_Aide:Position:=TEXTFILEPOS(FichierAide);
      tf_Print:Position:=TEXTFILEPOS(FichierPrint);
    end;
    Erreur:=ioresult;
    if Erreur = 0 then Ok:=true else Avorter:=not MESSAGE_ERREUR(Erreur);
  until (Ok) or (Avorter);
  POSITION_FICHIER:=Ok;
End;


Function DEPLACEMENT_FICHIER (TypeFichier : byte; Position : longint) : boolean;
(* DEPLACEMENT DE LA POSITION COURANTE DANS UN FICHIER (seek).
   RETOURNE true SI OK OU false SI ERREUR. *)
Var Ok,Avorter:boolean;
    Erreur:integer;
Begin
  Ok:=false; Avorter:=false;
  repeat
    case TypeFichier of
      tf_Aide:TEXTSEEK(FichierAide,Position);
      tf_Print:TEXTSEEK(FichierPrint,Position);
      tf_Index:seek(FichierIndex,Position);
    end;
    Erreur:=ioresult;
    if Erreur = 0 then Ok:=true else Avorter:=not MESSAGE_ERREUR(Erreur);
  until (Ok) or (Avorter);
  DEPLACEMENT_FICHIER:=Ok;
End;


Function LECTURE_FICHIER (TypeFichier : byte) : boolean;
(* LECTURE D'UN FICHIER.
   RETOURNE true SI OK OU false SI ERREUR. *)
Var Ok,Avorter:boolean;
    Erreur:integer;
Begin
  Ok:=false; Avorter:=false;
  repeat
    case TypeFichier of
      tf_Categories:read(FichierCategories,BufferCategories);
      tf_Joueurs:read(FichierJoueurs,BufferJoueurs);
      tf_Matches:read(FichierMatches,BufferMatches);
      tf_Ini:readln(FichierIni,BufferIni);
      tf_Aide:readln(FichierAide,BufferAide);
      tf_Print:readln(FichierPrint,BufferPrint);
      tf_Index:read(FichierIndex,BufferIndex);
    end;
    Erreur:=ioresult;
    if Erreur = 0 then Ok:=true else if TypeFichier = tf_Ini then Avorter:=true else Avorter:=not MESSAGE_ERREUR(Erreur);
  until (Ok) or (Avorter);
  LECTURE_FICHIER:=Ok;
End;


Function ECRITURE_FICHIER (TypeFichier : byte) : boolean;
(* ECRITURE D'UN FICHIER.
   RETOURNE true SI OK OU false SI ERREUR. *)
Var Ok,Avorter:boolean;
    Erreur:integer;
Begin
  Ok:=false; Avorter:=false;
  repeat
    case TypeFichier of
      tf_Categories:write(FichierCategories,BufferCategories);
      tf_Joueurs:write(FichierJoueurs,BufferJoueurs);
      tf_Matches:write(FichierMatches,BufferMatches);
      tf_Ini:writeln(FichierIni,BufferIni);
      tf_Print:writeln(FichierPrint,BufferPrint);
      tf_Index:write(FichierIndex,BufferIndex);
    end;
    Erreur:=ioresult;
    if Erreur = 0 then Ok:=true else if TypeFichier = tf_Ini then Avorter:=true else Avorter:=not MESSAGE_ERREUR(Erreur);
  until (Ok) or (Avorter);
  ECRITURE_FICHIER:=Ok;
End;


Function FERMETURE_FICHIER (TypeFichier : byte) : boolean;
(* FERMETURE D'UN FICHIER.
   RETOURNE true SI OK OU false SI ERREUR. *)
Var Ok,Avorter:boolean;
    Erreur:integer;
Begin
  Ok:=false; Avorter:=false;
  repeat
    case TypeFichier of
      tf_Categories:close(FichierCategories);
      tf_Joueurs:close(FichierJoueurs);
      tf_Matches:close(FichierMatches);
      tf_Ini:close(FichierIni);
      tf_Aide:close(FichierAide);
      tf_Print:close(FichierPrint);
      tf_Index:close(FichierIndex);
    end;
    Erreur:=ioresult;
    if Erreur = 0 then Ok:=true else if TypeFichier = tf_Ini then Avorter:=true else Avorter:=not MESSAGE_ERREUR(Erreur);
  until (Ok) or (Avorter);
  FERMETURE_FICHIER:=Ok;
End;


Function FIN_DE_FICHIER (TypeFichier : byte) : boolean;
(* DETECTION DE LA FIN D'UN FICHIER (eof) *)
Begin
  case TypeFichier of
    tf_Aide:FIN_DE_FICHIER:=eof(FichierAide);
    tf_Print:FIN_DE_FICHIER:=eof(FichierPrint);
  end;
End;


Procedure LECTURE_FICHIER_INI;
(* LECTURE DES OPTIONS DANS LE FICHIER TTO.INI ET CHARGEMENT DES OPTIONS PAR DEFAUT DU PROGRAMME *)
Var Valeur:byte;
    Erreur:integer;
    Ok:boolean;
Begin
  assign(FichierIni,'TTO.INI'); NomFichierTraite:='TTO.INI';
  if OUVERTURE_FICHIER(tf_Ini)
     then
       begin
         Ok:=true;
         while (Ok) and (not eof(FichierIni)) do
           begin
             Ok:=LECTURE_FICHIER(tf_Ini);
             if Ok
                then
                  if copy(BufferIni,1,12) = 'MouseDriver='
                     then   (* PAR DEFAUT : SI DRIVER SOURIS TROUVE ALORS ACTIVE *)
                       Souris:=(copy(BufferIni,13,2) = 'ON')
                     else
                  if copy(BufferIni,1,14) = 'AutoSaveDelay='
                     then   (* PAR DEFAUT : AUTO-SAVE = 15 MINUTES *)
                       begin
                         val(copy(BufferIni,15,2),Valeur,Erreur);
                         if Erreur = 0 then DelaiSauvetage:=Valeur else DelaiSauvetage:=15;
                       end
                     else
                  if copy(BufferIni,1,8) = 'Printer='
                     then   (* PAR DEFAUT : PAS D'IMPRIMANTE INSTALLEE *)
                       begin
                         if copy(BufferIni,9,9) = 'HPDESKJET' then Imprimante:=ti_HPDeskJet else
                         if copy(BufferIni,9,10) = 'PROPRINTER' then Imprimante:=ti_ProPrinter else
                         if copy(BufferIni,9,7) = 'EPSONLX' then Imprimante:=ti_EpsonLX else
                         if copy(BufferIni,9,11) = 'USERDEFINED' then Imprimante:=ti_Autre;
                       end
                     else
                  if copy(BufferIni,1,13) = 'CompressedOn='
                     then   (* CODE D'IMPRESSION EN COMPRESSE *)
                       ci_CompressedON:=ASCII_TO_CHAR(copy(BufferIni,14,length(BufferIni)-13))
                     else
                  if copy(BufferIni,1,14) = 'CompressedOff='
                     then   (* CODE D'IMPRESSION EN LARGEUR NORMALE *)
                       ci_CompressedOFF:=ASCII_TO_CHAR(copy(BufferIni,15,length(BufferIni)-14))
                     else
                  if copy(BufferIni,1,7) = 'BoldOn='
                     then   (* CODE D'IMPRESSION EN GRAS *)
                       ci_BoldON:=ASCII_TO_CHAR(copy(BufferIni,8,length(BufferIni)-7))
                     else
                  if copy(BufferIni,1,8) = 'BoldOff='
                     then   (* CODE D'IMPRESSION SANS GRAS *)
                       ci_BoldOFF:=ASCII_TO_CHAR(copy(BufferIni,9,length(BufferIni)-8))
                     else
                  if copy(BufferIni,1,12) = 'UnderlineOn='
                     then   (* CODE D'IMPRESSION EN SOULIGNE *)
                       ci_UnderlineON:=ASCII_TO_CHAR(copy(BufferIni,13,length(BufferIni)-12))
                     else
                  if copy(BufferIni,1,13) = 'UnderlineOff='
                     then   (* CODE D'IMPRESSION SANS SOULIGNE *)
                       ci_UnderlineOFF:=ASCII_TO_CHAR(copy(BufferIni,14,length(BufferIni)-13))
                     else
                  if copy(BufferIni,1,12) = 'SaveOptions='
                     then   (* PAR DEFAUT : SAUVER LES OPTIONS EN FIN DE PROGRAMME *)
                       SauverOptions:=(copy(BufferIni,13,2) = 'ON')
                     else
                  if copy(BufferIni,1,16) = 'ScreenSaveDelay='
                     then   (* PAR DEFAUT : ECONOMISEUR D'ECRAN TOUTES LES 3 MINUTES *)
                       begin
                         val(copy(BufferIni,17,2),Valeur,Erreur);
                         if Erreur = 0 then DelaiEcran:=Valeur else DelaiEcran:=3;
                       end;
           end;
         Void:=FERMETURE_FICHIER(tf_Ini);
       end;
  (* MISE A JOUR DES CODES D'IMPRIMANTE *)
  case Imprimante of
    ti_HPDeskJet:begin
                   ci_CompressedON:=ci_HP_CompressedON;
                   ci_CompressedOFF:=ci_HP_CompressedOFF;
                   ci_BoldON:=ci_HP_BoldON;
                   ci_BoldOFF:=ci_HP_BoldOFF;
                   ci_UnderlineON:=ci_HP_UnderlineON;
                   ci_UnderlineOFF:=ci_HP_UnderlineOFF;
                 end;
    ti_Proprinter:begin
                    ci_CompressedON:=ci_IBM_CompressedON;
                    ci_CompressedOFF:=ci_IBM_CompressedOFF;
                    ci_BoldON:=ci_IBM_BoldON;
                    ci_BoldOFF:=ci_IBM_BoldOFF;
                    ci_UnderlineON:=ci_IBM_UnderlineON;
                    ci_UnderlineOFF:=ci_IBM_UnderlineOFF;
                  end;
    ti_EpsonLX:begin
                 ci_CompressedON:=ci_Epson_CompressedON;
                 ci_CompressedOFF:=ci_Epson_CompressedOFF;
                 ci_BoldON:=ci_Epson_BoldON;
                 ci_BoldOFF:=ci_Epson_BoldOFF;
                 ci_UnderlineON:=ci_Epson_UnderlineON;
                 ci_UnderlineOFF:=ci_Epson_UnderlineOFF;
               end;
  end;
End;


Procedure ECRITURE_FICHIER_INI;
(* EN FIN DE PROGRAMME, ECRITURE DES OPTIONS DANS TTO.INI ET BACKUP TTO.INI --> TTO.I~I *)
Var Conversion:string[2];
    Ok:boolean;
Begin
  assign(FichierIni,'TTO.I~I'); erase(FichierIni);
  assign(FichierIni,'TTO.INI'); rename(FichierIni,'TTO.I~I');
  assign(FichierIni,'TTO.INI'); NomFichierTraite:='TTO.INI';
  if CREATION_FICHIER(tf_Ini)
     then
       begin
         BufferIni:='Version=1.20';
         Ok:=ECRITURE_FICHIER(tf_Ini);
         if Ok
            then
              begin
                if Souris then BufferIni:='MouseDriver=ON' else BufferIni:='MouseDriver=OFF';
                Ok:=ECRITURE_FICHIER(tf_Ini);
              end;
         if Ok
            then
              begin
                str(DelaiSauvetage,Conversion); if DelaiSauvetage < 10 then insert('0',Conversion,1);
                BufferIni:='AutoSaveDelay=' + Conversion;
                Ok:=ECRITURE_FICHIER(tf_Ini);
              end;
         if Ok
            then
              begin
                str(DelaiEcran,Conversion); if DelaiEcran < 10 then insert('0',Conversion,1);
                BufferIni:='ScreenSaveDelay=' + Conversion;
                Ok:=ECRITURE_FICHIER(tf_Ini);
              end;
         if Ok
            then
              begin
                case Imprimante of
                  ti_HPDeskJet:BufferIni:='Printer=HPDESKJET';
                  ti_ProPrinter:BufferIni:='Printer=PROPRINTER';
                  ti_EpsonLX:BufferIni:='Printer=EPSONLX';
                  ti_Autre:BufferIni:='Printer=USERDEFINED';
                  ti_Aucune:BufferIni:='Printer=NONE';
                end;
                Ok:=ECRITURE_FICHIER(tf_Ini);
              end;
         if (Ok) and (Imprimante = ti_Autre)
            then
              begin
                BufferIni:='CompressedOn=' + CHAR_TO_ASCII(ci_CompressedON);
                Ok:=ECRITURE_FICHIER(tf_Ini);
              end;
         if (Ok) and (Imprimante = ti_Autre)
            then
              begin
                BufferIni:='CompressedOff=' + CHAR_TO_ASCII(ci_CompressedOFF);
                Ok:=ECRITURE_FICHIER(tf_Ini);
              end;
         if (Ok) and (Imprimante = ti_Autre)
            then
              begin
                BufferIni:='BoldOn=' + CHAR_TO_ASCII(ci_BoldON);
                Ok:=ECRITURE_FICHIER(tf_Ini);
              end;
         if (Ok) and (Imprimante = ti_Autre)
            then
              begin
                BufferIni:='BoldOff=' + CHAR_TO_ASCII(ci_BoldOFF);
                Ok:=ECRITURE_FICHIER(tf_Ini);
              end;
         if (Ok) and (Imprimante = ti_Autre)
            then
              begin
                BufferIni:='UnderlineOn=' + CHAR_TO_ASCII(ci_UnderlineON);
                Ok:=ECRITURE_FICHIER(tf_Ini);
              end;
         if (Ok) and (Imprimante = ti_Autre)
            then
              begin
                BufferIni:='UnderlineOff=' + CHAR_TO_ASCII(ci_UnderlineOFF);
                Ok:=ECRITURE_FICHIER(tf_Ini);
              end;
         if Ok
            then
              begin
                if SauverOptions then BufferIni:='SaveOptions=ON' else BufferIni:='SaveOptions=OFF';
                Ok:=ECRITURE_FICHIER(tf_Ini);
              end;
         Ok:=FERMETURE_FICHIER(tf_Ini);
       end;
End;


Function SAUVETAGE_CATEGORIES : boolean;
(* SAUVETAGE DES CATEGORIES SUR DISQUE.
   RETOURNE true SI OK OU false SI ERREUR. *)
Var n:byte;
    Ok:boolean;
Begin
  NomFichierTraite:='CATEGORI.TTO'; assign(FichierCategories,NomFichierTraite);
  if CREATION_FICHIER(tf_Categories)
     then
       begin
         n:=1; Ok:=true;
         while (n <= 20) and (Ok) do
           begin
             BufferCategories.NomCategorie:=Categories[n].NomCategorie;
             BufferCategories.Situation:=Categories[n].Situation;
             BufferCategories.NbPoules:=Categories[n].NbPoules;
             BufferCategories.NbJoueurs:=Categories[n].NbJoueurs;
             BufferCategories.NbJoueursRestants:=Categories[n].NbJoueursRestants;
             BufferCategories.NbMatches:=Categories[n].NbMatches;
             BufferCategories.NbMatchesRestants:=Categories[n].NbMatchesRestants;
             if ECRITURE_FICHIER(tf_Categories) then inc(n) else Ok:=false;
           end;
         if Ok then Ok:=FERMETURE_FICHIER(tf_Categories);
       end
     else
       Ok:=false;
  SAUVETAGE_CATEGORIES:=Ok;
End;


Function SAUVETAGE_JOUEURS : boolean;
(* SAUVETAGE DES JOUEURS SUR DISQUE.
   RETOURNE true SI OK OU false SI ERREUR. *)
Var n:byte;
    Ok:boolean;
    p:pJoueur;
Begin
  NomFichierTraite:='JOUEURS.TTO'; assign(FichierJoueurs,NomFichierTraite);
  if CREATION_FICHIER(tf_Joueurs)
     then
       begin
         n:=1; Ok:=true;
         while (n <= 20) and (Ok) do
           begin
             p:=Categories[n].pTeteJoueurs;
             while (p <> nil) and (Ok) do
               begin
                 BufferJoueurs.Categorie:=n;
                 BufferJoueurs.NumeroSauvetage:=p^.NumeroSauvetage;
                 BufferJoueurs.NomJoueur:=p^.NomJoueur;
                 BufferJoueurs.PrenomJoueur:=p^.PrenomJoueur;
                 BufferJoueurs.Age:=p^.Age;
                 BufferJoueurs.Club:=p^.Club;
                 BufferJoueurs.EnLice:=p^.EnLice;
                 BufferJoueurs.Poule:=p^.Poule;
                 BufferJoueurs.MatchesJoues:=p^.MatchesJoues;
                 BufferJoueurs.Points:=p^.Points;
                 BufferJoueurs.Sets:=p^.Sets;
                 BufferJoueurs.Score:=p^.Score;
                 if ECRITURE_FICHIER(tf_Joueurs) then p:=p^.pSuiv else Ok:=false;
               end;
             inc(n);
           end;
         if Ok then Ok:=FERMETURE_FICHIER(tf_Joueurs);
       end
     else
       Ok:=false;
  SAUVETAGE_JOUEURS:=Ok;
End;


Function SAUVETAGE_MATCHES : boolean;
(* SAUVETAGE DES MATCHES SUR DISQUE.
   RETOURNE true SI OK OU false SI ERREUR. *)
Var n:byte;
    Ok:boolean;
    pJ:pJoueur;
    pM:pMatch;
    Numero1,Numero2:word;
Begin
  NomFichierTraite:='MATCHES.TTO'; assign(FichierMatches,NomFichierTraite);
  if CREATION_FICHIER(tf_Matches)
     then
       begin
         n:=1; Ok:=true;
         while (n <= 20) and (Ok) do
           begin
             pM:=Categories[n].pTeteMatches;
             while (pM <> nil) and (Ok) do
               begin
                 BufferMatches.Categorie:=n;
                 BufferMatches.Poule:=pM^.Poule;
                 pJ:=pM^.pJoueur1; BufferMatches.NumeroJoueur1:=pJ^.NumeroSauvetage;
                 pJ:=pM^.pJoueur2; BufferMatches.NumeroJoueur2:=pJ^.NumeroSauvetage;
                 BufferMatches.Resultats:=pM^.Resultats;
                 if ECRITURE_FICHIER(tf_Matches) then pM:=pM^.pSuiv else Ok:=false;
               end;
             inc(n);
           end;
         if Ok then Ok:=FERMETURE_FICHIER(tf_Matches);
       end
     else
       Ok:=false;
  SAUVETAGE_MATCHES:=Ok;
End;


Function SAUVETAGE_SITUATION (Option : byte) : boolean;
(* SAUVETAGE DE LA SITUATION SUR DISQUE.
   OPTION : ss_Normal : SAUVETAGE NORMAL,
            ss_Automatique : SAUVETAGE AUTOMATIQUE.
   RETOURNE true SI L'OPERATION S'EST BIEN DEROULEE OU false SI ERREUR. *)
Const c=9; l=9; nbl=6; lg=62;
Var x,y,x1,y1,x2,y2,FondCourant,CouleurCourante:word;
    Fond1,Normal1,Fond2,Brillant,Fond3,Normal3,FondOmbre,Ombre:word;
    Image:videosave;
    Ligne:byte;
    Numero:word;
    p:pJoueur;
    Entree:char;
    Ok:boolean;
Begin
  case ModeEcran of
    ModeMono:begin
               Fond1:=lightgrey; Normal1:=black; Fond2:=black; Brillant:=white;
               Fond3:=lightgrey; Normal3:=white; FondOmbre:=black; Ombre:=lightgrey;
             end;
    ModeCouleur:begin
                  Fond1:=green; Normal1:=yellow; Fond2:=magenta; Brillant:=white;
                  Fond3:=green; Normal3:=white; FondOmbre:=black; Ombre:=lightgrey;
                end;
  end;
  if (Option = ss_Automatique) and (Souris) then Void:=txtmousecursoff;
  x1:=windx1; y1:=windy1; x2:=windx2; y2:=windy2; x:=windx; y:=windy; FondCourant:=textbackgr; CouleurCourante:=textcolour;
  txtunwind; txtsave(Image);
  txtback(Fond1); txtcolour(Normal1);
  txtmove(c,l);   txtwrite(' ͻ ');
  txtmove(c,l+1); txtwrite('  . Sauvetage des catgories dans le fichier CATEGORI.TTO   ');
  txtmove(c,l+2); txtwrite('  . Sauvetage des joueurs dans le fichier JOUEURS.TTO       ');
  txtmove(c,l+3); txtwrite('  . Sauvetage des matches dans le fichier MATCHES.TTO       ');
  txtmove(c,l+4); txtwrite('                       <            >                       ');
  txtmove(c,l+5); txtwrite(' ͼ ');
  txtback(FondOmbre); txtcolour(Ombre);
  for Ligne:=1 to (nbl-1) do begin txtmove(c+lg,l+Ligne); txtattrib(2); end;
  txtmove(c+2,l+nbl); txtattrib(lg);
  txtback(Fond3); txtcolour(Normal3); txtmove(c+25,l+4); txtwrite('    (O)k    ');
  txtback(Fond1); txtcolour(Brillant);
  if Option = ss_Normal
     then
       begin
         txtmove(29,l); txtwrite(' Sauvetage sur disque '); TITRE_HAUT_ECRAN(5);
       end
     else
       begin
         txtmove(23,l); txtwrite(' Sauvetage automatique sur disque ');
       end;
  (* NUMEROTATION DES JOUEURS (POUR LIENS JOUEURS-MATCHES) *)
  for Ligne:=1 to 20 do
    begin
      Numero:=1; p:=Categories[Ligne].pTeteJoueurs;
      while p <> nil do begin p^.NumeroSauvetage:=Numero; inc(Numero); p:=p^.pSuiv; end;
    end;
  SAUVETAGE_SITUATION:=true;
  (* SAUVETAGE DES CATEGORIES *)
  if Option = ss_Normal then MESSAGE_BAS_ECRAN(ip_SauvetageDisque + 1);
  txtback(Fond2); txtcolour(Brillant); txtmove(c+3,l+1); txtwrite(#16);
  if SAUVETAGE_CATEGORIES
     then
       begin
         txtback(Fond3); txtcolour(Normal3); txtmove(c+3,l+1); txtwrite('');
         (* SAUVETAGE DES JOUEURS *)
         if Option = ss_Normal then MESSAGE_BAS_ECRAN(ip_SauvetageDisque + 2);
         txtback(Fond2); txtcolour(Brillant); txtmove(c+3,l+2); txtwrite(#16);
         if SAUVETAGE_JOUEURS
            then
              begin
                txtback(Fond3); txtcolour(Normal3); txtmove(c+3,l+2); txtwrite('');
                (* SAUVETAGE DES MATCHES *)
                if Option = ss_Normal then MESSAGE_BAS_ECRAN(ip_SauvetageDisque + 3);
                txtback(Fond2); txtcolour(Brillant); txtmove(c+3,l+3); txtwrite(#16);
                if SAUVETAGE_MATCHES
                   then
                     begin
                       txtback(Fond3); txtcolour(Normal3); txtmove(c+3,l+3); txtwrite('');
                     end
                   else
                     SAUVETAGE_SITUATION:=false;
              end
            else
              SAUVETAGE_SITUATION:=false;
       end
     else
       SAUVETAGE_SITUATION:=false;
  if Option = ss_Normal
     then   (* BOUTON "OK" *)
       begin
         txtback(Fond2); txtcolour(Brillant); txtmove(c+25,l+4); txtattrib(12);
         MESSAGE_BAS_ECRAN(ip_SauvetageDisque);
         txtflush; mouseeventparams.me_modified:=false;
         Ok:=false; if Souris then Void:=txtmousecurson;
         repeat
           if (DelaiEcran <> 0) and (EconomiserEcran) then ECONOMISEUR_ECRAN;
           if txtkeypressed
              then
                begin
                  txtinput(Entree);
                  case ord(Entree) of
                    0:begin
                        txtinput(Entree); if ord(Entree) = 24 then Ok:=true;   (* ALT-O *)
                      end;
                    13,79,111:Ok:=true;   (* ENTER / "O","o" : OK *)
                  end;
                end;
           if mouseeventparams.me_modified
              then
                with mouseeventparams do
                  begin
                    if (me_y = (l+4)) and (me_x in [(c+25)..(c+36)]) then Ok:=true;
                    me_modified:=false;
                  end;
         until Ok;
         if Souris then Void:=txtmousecursoff;
       end;
  (* REMISE A ZERO DU COMPTEUR POUR LE SAUVETAGE AUTOMATIQUE *)
  CompteurTicsSauvetage:=0; SauverSituation:=false;
  txtunwind; txtload(Image); txtwind(x1,y1,x2,y2); txtback(FondCourant); txtcolour(CouleurCourante); txtmove(x,y);
  if (Option = ss_Automatique) and (Souris) then Void:=txtmousecurson;
End;


Function RECUPERATION_CATEGORIES : boolean;
(* RECUPERATION DES CATEGORIES SUR DISQUE.
   RETOURNE true SI OK OU false SI ERREUR. *)
Var n:byte;
    Ok:boolean;
Begin
  NomFichierTraite:='CATEGORI.TTO'; assign(FichierCategories,NomFichierTraite);
  if OUVERTURE_FICHIER(tf_Categories)
     then
       begin
         n:=1; Ok:=true;
         while (n <= 20) and (Ok) do
           if LECTURE_FICHIER(tf_Categories)
              then
                begin
                  Categories[n].NomCategorie:=BufferCategories.NomCategorie;
                  Categories[n].Situation:=BufferCategories.Situation;
                  Categories[n].NbPoules:=BufferCategories.NbPoules;
                  Categories[n].NbJoueurs:=BufferCategories.NbJoueurs;
                  Categories[n].NbJoueursRestants:=BufferCategories.NbJoueursRestants;
                  Categories[n].NbMatches:=BufferCategories.NbMatches;
                  Categories[n].NbMatchesRestants:=BufferCategories.NbMatchesRestants;
                  inc(n);
                end
              else
                Ok:=false;
         Void:=FERMETURE_FICHIER(tf_Categories);
       end
     else
       Ok:=false;
  RECUPERATION_CATEGORIES:=Ok;
End;


Function RECUPERATION_JOUEURS : boolean;
(* RECUPERATION DES JOUEURS SUR DISQUE.
   RETOURNE true SI OK OU false SI ERREUR. *)
Var Ok:boolean;
    p:pJoueur;
Begin
  NomFichierTraite:='JOUEURS.TTO'; assign(FichierJoueurs,NomFichierTraite);
  if OUVERTURE_FICHIER(tf_Joueurs)
     then
       begin
         Ok:=true;
         while (not eof(FichierJoueurs)) and (Ok) do
           if LECTURE_FICHIER(tf_Joueurs)
              then
                begin
                  new(p);
                  if p <> nil
                     then
                       begin
                         p^.NumeroSauvetage:=BufferJoueurs.NumeroSauvetage;
                         p^.NomJoueur:=BufferJoueurs.NomJoueur;
                         p^.PrenomJoueur:=BufferJoueurs.PrenomJoueur;
                         p^.Age:=BufferJoueurs.Age;
                         p^.Club:=BufferJoueurs.Club;
                         p^.EnLice:=BufferJoueurs.EnLice;
                         p^.Poule:=BufferJoueurs.Poule;
                         p^.MatchesJoues:=BufferJoueurs.MatchesJoues;
                         p^.Points:=BufferJoueurs.Points;
                         p^.Sets:=BufferJoueurs.Sets;
                         p^.Score:=BufferJoueurs.Score;
                         p^.pPrec:=nil; p^.pSuiv:=nil;
                         AJOUT_JOUEUR_LISTE_CATEGORIE(BufferJoueurs.Categorie,list_Ajout,p);
                       end
                     else   (* ERREUR D'ALLOCATION MEMOIRE *)
                       begin
                         Void:=MESSAGE_ERREUR(1001); Ok:=false;
                       end;
                end
              else
                Ok:=false;
         Void:=FERMETURE_FICHIER(tf_Joueurs);
       end
     else
       Ok:=false;
  RECUPERATION_JOUEURS:=Ok;
End;


Function RECUPERATION_MATCHES : boolean;
(* RECUPERATION DES MATCHES SUR DISQUE.
   RETOURNE true SI OK OU false SI ERREUR. *)
Var Ok,Trouve:boolean;
    pJ:pJoueur;
    pM:pMatch;
    Numero1,Numero2:word;
Begin
  NomFichierTraite:='MATCHES.TTO'; assign(FichierMatches,NomFichierTraite);
  if OUVERTURE_FICHIER(tf_Matches)
     then
       begin
         Ok:=true;
         while (not eof(FichierMatches)) and (Ok) do
           if LECTURE_FICHIER(tf_Matches)
              then
                begin
                  new(pM);
                  if pM <> nil
                     then
                       begin
                         pM^.Poule:=BufferMatches.Poule;
                         Numero1:=BufferMatches.NumeroJoueur1; Numero2:=BufferMatches.NumeroJoueur2;
                         pM^.Resultats:=BufferMatches.Resultats;
                         pM^.pPrec:=nil; pM^.pSuiv:=nil;
                         AJOUT_MATCH_LISTE_CATEGORIE(BufferMatches.Categorie,pM);
                         (* RETABLISSEMENT LIENS AVEC JOUEURS (VIA NumeroSauvetage) *)
                         (* JOUEUR 1 *)
                         pJ:=Categories[BufferMatches.Categorie].pTeteJoueurs; Trouve:=false;
                         while (pJ <> nil) and (not Trouve) do
                           begin
                             if pJ^.NumeroSauvetage = Numero1 then Trouve:=true else pJ:=pJ^.pSuiv;
                           end;
                         if Trouve
                            then   (* JOUEUR 2 *)
                              begin
                                pM^.pJoueur1:=pJ;
                                pJ:=Categories[BufferMatches.Categorie].pTeteJoueurs; Trouve:=false;
                                while (pJ <> nil) and (not Trouve) do
                                  begin
                                    if pJ^.NumeroSauvetage = Numero2 then Trouve:=true else pJ:=pJ^.pSuiv;
                                  end;
                                if Trouve then pM^.pJoueur2:=pJ;
                              end;
                         if not Trouve
                            then   (* ERREUR LORS DU RETABLISSEMENT DES LIENS JOUEURS <--> MATCHES *)
                              begin
                                Void:=MESSAGE_ERREUR(1002); Ok:=false;
                              end;
                       end
                     else   (* ERREUR D'ALLOCATION MEMOIRE *)
                       begin
                         Void:=MESSAGE_ERREUR(1001); Ok:=false;
                       end;
                end
              else
                Ok:=false;
         Void:=FERMETURE_FICHIER(tf_Matches);
       end
     else
       Ok:=false;
  RECUPERATION_MATCHES:=Ok;
End;


Function RECUPERATION_SITUATION : boolean;
(* RECUPERATION DE LA SITUATION SAUVEE SUR DISQUE.
   RETOURNE true SI L'OPERATION S'EST BIEN DEROULEE OU false SI ERREUR. *)
Const c=8; l=9; nbl=6; lg=64;
Var x,y,x1,y1,x2,y2,FondCourant,CouleurCourante:word;
    Fond1,Normal1,Fond2,Brillant,Fond3,Normal3,FondOmbre,Ombre:word;
    Image:videosave;
    Ligne:byte;
    Numero:word;
    p:pJoueur;
    Entree:char;
    Ok:boolean;
Begin
  case ModeEcran of
    ModeMono:begin
               Fond1:=lightgrey; Normal1:=black; Fond2:=black; Brillant:=white;
               Fond3:=lightgrey; Normal3:=white; FondOmbre:=black; Ombre:=lightgrey;
             end;
    ModeCouleur:begin
                  Fond1:=green; Normal1:=yellow; Fond2:=magenta; Brillant:=white;
                  Fond3:=green; Normal3:=white; FondOmbre:=black; Ombre:=lightgrey;
                end;
  end;
  x1:=windx1; y1:=windy1; x2:=windx2; y2:=windy2; x:=windx; y:=windy; FondCourant:=textbackgr; CouleurCourante:=textcolour;
  txtunwind; txtsave(Image);
  txtback(Fond1); txtcolour(Normal1);
  txtmove(c,l);   txtwrite(' ͻ ');
  txtmove(c,l+1); txtwrite('  . Rcupration des catgories dans le fichier CATEGORI.TTO  ');
  txtmove(c,l+2); txtwrite('  . Rcupration des joueurs dans le fichier JOUEURS.TTO      ');
  txtmove(c,l+3); txtwrite('  . Rcupration des matches dans le fichier MATCHES.TTO      ');
  txtmove(c,l+4); txtwrite('                        <            >                        ');
  txtmove(c,l+5); txtwrite(' ͼ ');
  txtback(FondOmbre); txtcolour(Ombre);
  for Ligne:=1 to (nbl-1) do begin txtmove(c+lg,l+Ligne); txtattrib(2); end;
  txtmove(c+2,l+nbl); txtattrib(lg);
  txtback(Fond3); txtcolour(Normal3); txtmove(c+26,l+4); txtwrite('    (O)k    ');
  txtback(Fond1); txtcolour(Brillant); txtmove(27,l); txtwrite(' Rcupration sur disque ');
  (* VIDANGE DES LISTES CHAINEES EN MEMOIRE ET REINITIALISATION DES VARIABLES *)
  for Ligne:=1 to 20 do
    with Categories[Ligne] do
      begin
        if pTeteJoueurs <> nil
           then
             begin
               SUPPRESSION_LISTE_JOUEURS_CATEGORIE(Ligne); pTeteJoueurs:=nil; pQueueJoueurs:=nil;
             end;
        if pTeteMatches <> nil
           then
             begin
               SUPPRESSION_LISTE_MATCHES_CATEGORIE(Ligne); pTeteMatches:=nil; pQueueMatches:=nil;
             end;
        NomCategorie:=''; Situation:=0; NbPoules:=0; NbJoueurs:=0; NbJoueursRestants:=0; NbMatches:=0; NbMatchesRestants:=0;
      end;
  RECUPERATION_SITUATION:=true;
  (* RECUPERATION DES CATEGORIES *)
  MESSAGE_BAS_ECRAN(ip_RecuperationDisque + 1);
  txtback(Fond2); txtcolour(Brillant); txtmove(c+3,l+1); txtwrite(#16);
  if RECUPERATION_CATEGORIES
     then
       begin
         txtback(Fond3); txtcolour(Normal3); txtmove(c+3,l+1); txtwrite('');
         (* SAUVETAGE DES JOUEURS *)
         MESSAGE_BAS_ECRAN(ip_RecuperationDisque + 2);
         txtback(Fond2); txtcolour(Brillant); txtmove(c+3,l+2); txtwrite(#16);
         if RECUPERATION_JOUEURS
            then
              begin
                txtback(Fond3); txtcolour(Normal3); txtmove(c+3,l+2); txtwrite('');
                (* SAUVETAGE DES MATCHES *)
                MESSAGE_BAS_ECRAN(ip_RecuperationDisque + 3);
                txtback(Fond2); txtcolour(Brillant); txtmove(c+3,l+3); txtwrite(#16);
                if RECUPERATION_MATCHES
                   then
                     begin
                       txtback(Fond3); txtcolour(Normal3); txtmove(c+3,l+3); txtwrite('');
                     end
                   else
                     RECUPERATION_SITUATION:=false;
              end
            else
              RECUPERATION_SITUATION:=false;
       end
     else
       RECUPERATION_SITUATION:=false;
  (* BOUTON "OK" *)
  txtback(Fond2); txtcolour(Brillant); txtmove(c+26,l+4); txtattrib(12);
  MESSAGE_BAS_ECRAN(ip_SauvetageDisque);
  txtflush; mouseeventparams.me_modified:=false;
  Ok:=false; if Souris then Void:=txtmousecurson;
  repeat
    if (DelaiEcran <> 0) and (EconomiserEcran) then ECONOMISEUR_ECRAN;
    if txtkeypressed
       then
         begin
           txtinput(Entree);
           case ord(Entree) of
             0:begin
                 txtinput(Entree); if ord(Entree) = 24 then Ok:=true;   (* ALT-O *)
               end;
             13,79,111:Ok:=true;   (* ENTER / "O","o" : OK *)
           end;
         end;
    if mouseeventparams.me_modified
       then
         with mouseeventparams do
           begin
             if (me_y = (l+4)) and (me_x in [(c+26)..(c+37)]) then Ok:=true;
             me_modified:=false;
           end;
  until Ok;
  if Souris then Void:=txtmousecursoff;
  txtunwind; txtload(Image); txtwind(x1,y1,x2,y2); txtback(FondCourant); txtcolour(CouleurCourante); txtmove(x,y);
End;


Function CREATION_FICHIER_INDEX (TypeFichier, Numero : byte) : boolean;
(* CREATION D'UN FICHIER D'INDEX POUR L'AFFICHAGE D'UN FICHIER TEXTE.
   PRINCIPE : LE FICHIER TEXTE EST LU ENTIEREMENT. A CHAQUE LIGNE, LA POSITION
              COURANTE DANS LE FICHIER EST ECRITE DANS LE FICHIER INDEX.
   - DANS LE CAS DU FICHIER TTO.HLP, LE FICHIER TTOHLP.NDX EST CREE.
   - DANS LE CAS D'UN FICHIER D'IMPRESSION (CAT??.PRN), LE FICHIER TTOPRN.NDX EST CREE.
   LA TOUCHE <ESCAPE> PEUT INTERROMPRE LE PROCESSUS.
   RETOURNE true SI TOUT S'EST BIEN PASSE ET false DANS LE CAS CONTRAIRE. *)
Const c=12; l=11; lg=56;
Var x,y,x1,y1,x2,y2,FondCourant,CouleurCourante:word;
    Image:videosave;
    Fond1,Normal1,Brillant,FondOmbre,Ombre:word;
    Chaine,Source,Dest:string[12];
    Position:longint;
    Ok:boolean;
    Entree:char;
Begin
  case ModeEcran of
    ModeMono:begin Fond1:=lightgrey; Normal1:=black; Brillant:=white; FondOmbre:=black; Ombre:=lightgrey; end;
    ModeCouleur:begin Fond1:=green; Normal1:=yellow; Brillant:=white; FondOmbre:=black; Ombre:=lightgrey; end;
  end;
  (* CADRE *)
  x1:=windx1; y1:=windy1; x2:=windx2; y2:=windy2; x:=windx; y:=windy; FondCourant:=textbackgr; CouleurCourante:=textcolour;
  txtunwind; txtsave(Image);
  txtback(Fond1); txtcolour(Normal1);
  txtmove(c,l);   txtwrite(' ͻ ');
  txtmove(c,l+1); txtwrite('        Cration du fichier d''index en cours...       ');
  txtmove(c,l+2); txtwrite(' ͼ ');
  txtback(FondOmbre); txtcolour(Ombre);
  txtmove(c+lg,l+1); txtattrib(2); txtmove(c+lg,l+2); txtattrib(2);
  txtmove(c+2,l+3); txtattrib(lg);
  txtback(Fond1); txtcolour(Brillant); txtmove(31,l); txtwrite(' Fichier d''index ');
  MESSAGE_BAS_ECRAN(ip_Indexation + 1);
  (* CREATION DU FICHIER INDEX *)
  case TypeFichier of
    tf_Aide:begin
              Source:='TTO.HLP'; assign(FichierAide,Source);
              Dest:='TTOHLP.NDX'; assign(FichierIndex,Dest);
            end;
    tf_Print:begin
               str(Numero,Chaine); if length(Chaine) = 1 then insert('0',Chaine,1);
               Source:='CAT' + Chaine + '.PRN'; assign(FichierPrint,Source);
               Dest:='TTOPRN.NDX'; assign(FichierIndex,Dest);
             end;
  end;
  Ok:=true;
  (* OUVERTURE DU FICHIER SOURCE *)
  NomFichierTraite:=Source;
  if OUVERTURE_FICHIER(TypeFichier)
     then
       begin
         (* CREATION DU FICHIER INDEX *)
         NomFichierTraite:=Dest;
         if CREATION_FICHIER(tf_Index)
            then
              begin
                (* BOUCLE DE DETERMINATION DES INDEX *)
                txtflush;
                while (not FIN_DE_FICHIER(TypeFichier)) and (Ok) do
                  begin
                    if txtkeypressed
                       then   (* TOUCHE DU CLAVIER DETECTEE *)
                         begin
                           txtinput(Entree);
                           case ord(Entree) of
                             0:txtinput(Entree);
                             27:Ok:=false;   (* INTERRUPTION DU PROCESSUS *)
                           end;
                         end;
                    if Ok
                       then
                         begin
                           (* DETERMINATION DE LA POSITION DANS LE FICHIER SOURCE *)
                           NomFichierTraite:=Source;
                           if POSITION_FICHIER(TypeFichier,BufferIndex)
                              then
                                begin
                                  (* ECRITURE DE LA POSITION DANS LE FICHIER INDEX *)
                                  NomFichierTraite:=Dest;
                                  if ECRITURE_FICHIER(tf_Index)
                                     then
                                       begin
                                         (* LECTURE D'UNE LIGNE DU FICHIER SOURCE *)
                                         NomFichierTraite:=Source;
                                         Ok:=LECTURE_FICHIER(TypeFichier)
                                       end
                                     else
                                       Ok:=false;
                                end
                              else
                                Ok:=false;
                         end;
                  end;
                (* FERMETURE DU FICHIER INDEX *)
                Void:=FERMETURE_FICHIER(tf_Index); if Ok then Ok:=Void;
              end
            else
              Ok:=false;
         (* FERMETURE DU FICHIER SOURCE *)
         Void:=FERMETURE_FICHIER(TypeFichier);
       end
     else
       Ok:=false;
  CREATION_FICHIER_INDEX:=Ok;
  (* RESTAURATION DE L'ECRAN DE DEPART *)
  txtunwind; txtload(Image); txtwind(x1,y1,x2,y2); txtback(FondCourant); txtcolour(CouleurCourante); txtmove(x,y);
End;


Function CHARGEMENT_INDEX (TypeFichier : byte) : boolean;
(* LECTURE D'UN FICHIER INDEX ET CHARGEMENT DU TABLEAU Texte.
   TypeFichier : tf_Help : CHARGEMENT DU FICHIER TTOHLP.NDX;
                 tf_Print : CHARGEMENT DU FICHIER TTOPRN.NDX.
   LA TOUCHE <ESCAPE> PEUT INTERROMPRE LE PROCESSUS.
   RETOURNE true SI LE CHARGEMENT S'EST BIEN EFFECTUE ET false SI ERREUR. *)
Const c=12; l=11; lg=56;
Var x,y,x1,y1,x2,y2,FondCourant,CouleurCourante:word;
    Image:videosave;
    Fond1,Normal1,Brillant,FondOmbre,Ombre:word;
    Indice:word;
    Ok:boolean;
    Entree:char;
Begin
  case ModeEcran of
    ModeMono:begin Fond1:=lightgrey; Normal1:=black; Brillant:=white; FondOmbre:=black; Ombre:=lightgrey; end;
    ModeCouleur:begin Fond1:=green; Normal1:=yellow; Brillant:=white; FondOmbre:=black; Ombre:=lightgrey; end;
  end;
  (* CADRE *)
  x1:=windx1; y1:=windy1; x2:=windx2; y2:=windy2; x:=windx; y:=windy; FondCourant:=textbackgr; CouleurCourante:=textcolour;
  txtunwind; txtsave(Image);
  txtback(Fond1); txtcolour(Normal1);
  txtmove(c,l);   txtwrite(' ͻ ');
  txtmove(c,l+1); txtwrite('        Lecture du fichier d''index en cours...        ');
  txtmove(c,l+2); txtwrite(' ͼ ');
  txtback(FondOmbre); txtcolour(Ombre);
  txtmove(c+lg,l+1); txtattrib(2); txtmove(c+lg,l+2); txtattrib(2);
  txtmove(c+2,l+3); txtattrib(lg);
  txtback(Fond1); txtcolour(Brillant); txtmove(31,l); txtwrite(' Fichier d''index ');
  MESSAGE_BAS_ECRAN(ip_Indexation + 1);
  (* LECTURE DU FICHIER INDEX *)
  case TypeFichier of
    tf_Aide:NomFichierTraite:='TTOHLP.NDX';
    tf_Print:NomFichierTraite:='TTOPRN.NDX';
  end;
  assign(FichierIndex,NomFichierTraite);
  Ok:=true; NbLignesTexte:=0;
  if OUVERTURE_FICHIER(tf_Index)
     then
       begin
         txtflush;
         while (not eof(FichierIndex)) and (Ok) do
           begin
             if txtkeypressed
                then   (* TOUCHE DU CLAVIER DETECTEE *)
                  begin
                    txtinput(Entree);
                    case ord(Entree) of
                      0:txtinput(Entree);
                      27:Ok:=false;   (* INTERRUPTION DU PROCESSUS *)
                    end;
                  end;
             if Ok
                then   (* LECTURE DU FICHIER *)
                  if LECTURE_FICHIER(tf_Index)
                     then
                       begin
                         inc(NbLignesTexte); Texte[NbLignesTexte]:=BufferIndex;
                       end
                     else
                       Ok:=false;
           end;
         Void:=FERMETURE_FICHIER(tf_Index);
       end
     else
       Ok:=false;
  CHARGEMENT_INDEX:=Ok;
  (* RESTAURATION DE L'ECRAN DE DEPART *)
  txtunwind; txtload(Image); txtwind(x1,y1,x2,y2); txtback(FondCourant); txtcolour(CouleurCourante); txtmove(x,y);
End;


Function LIRE_DATE_HEURE_FICHIER (TypeFichier : byte) : longint;
(* DETERMINE LA DATE ET L'HEURE D'UN FICHIER (QUI DOIT ETRE OUVERT !).
   RETOURNE LA DATE ET L'HEURE SOUS FORME COMPACTEE, OU 0 SI ERREUR. *)
Var Ok,Avorter:boolean;
    Erreur:integer;
    DateHeure:longint;
Begin
  Ok:=false; Avorter:=false;
  repeat
    case TypeFichier of
      tf_Aide:getftime(FichierAide,DateHeure);
      tf_Index:getftime(FichierIndex,DateHeure);
    end;
    Erreur:=doserror;
    if Erreur = 0 then Ok:=true else Avorter:=not MESSAGE_ERREUR(Erreur);
  until (Ok) or (Avorter);
  if Avorter then DateHeure:=0; LIRE_DATE_HEURE_FICHIER:=DateHeure;
End;


Function ECRIRE_DATE_HEURE_FICHIER (TypeFichier : byte; DateHeure : longint) : boolean;
(* ECRIT LA DATE ET L'HEURE D'UN FICHIER (QUI DOIT ETRE OUVERT !).
   RETOURNE true SI OK OU false SI ERREUR. *)
Var Ok,Avorter:boolean;
    Erreur:integer;
Begin
  Ok:=false; Avorter:=false;
  repeat
    case TypeFichier of
      tf_Index:setftime(FichierIndex,DateHeure);
    end;
    Erreur:=doserror;
    if Erreur = 0 then Ok:=true else Avorter:=not MESSAGE_ERREUR(Erreur);
  until (Ok) or (Avorter);
  ECRIRE_DATE_HEURE_FICHIER:=Ok;
End;


Function TEST_DATE_HEURE_FICHIERS : byte;
(* TESTE SI LES DATE ET HEURE DES FICHIERS TTO.HLP ET TTOHLP.NDX SONT IDENTIQUES.
   VALEURS RENVOYEES : tdh_Ok = OK;
                       tdh_Difference = DIFFERENCE;
                       tdh_Erreur = ERREUR. *)
Var DateHeureHLP,DateHeureNDX:longint;
Begin
  (* LECTURE DE LA DATE ET DE L'HEURE DU FICHIER TTO.HLP *)
  NomFichierTraite:='TTO.HLP'; assign(FichierAide,NomFichierTraite);
  if OUVERTURE_FICHIER(tf_Aide)
     then
       begin
         DateHeureHLP:=LIRE_DATE_HEURE_FICHIER(tf_Aide);
         Void:=FERMETURE_FICHIER(tf_Aide);
         if DateHeureHLP <> 0
            then
              begin
                (* LECTURE DE LA DATE ET DE L'HEURE DU FICHIER TTOHLP.NDX *)
                NomFichierTraite:='TTOHLP.NDX'; assign(FichierIndex,NomFichierTraite);
                if OUVERTURE_FICHIER(tf_Index)
                   then
                     begin
                       DateHeureNDX:=LIRE_DATE_HEURE_FICHIER(tf_Index);
                       Void:=FERMETURE_FICHIER(tf_Index);
                       if DateHeureNDX <> 0
                          then   (* TEST D'EGALITE *)
                            begin
                              if DateHeureHLP = DateHeureNDX
                                 then TEST_DATE_HEURE_FICHIERS:=tdh_Ok
                                 else TEST_DATE_HEURE_FICHIERS:=tdh_Difference;
                            end
                          else
                            TEST_DATE_HEURE_FICHIERS:=tdh_Erreur;
                     end
                   else
                     TEST_DATE_HEURE_FICHIERS:=tdh_Erreur;
              end
            else
              TEST_DATE_HEURE_FICHIERS:=tdh_Erreur;
       end
     else
       TEST_DATE_HEURE_FICHIERS:=tdh_Erreur;
End;


Function ALIGNER_DATE_HEURE_FICHIERS : boolean;
(* ATTRIBUE AU FICHIER TTOHLP.NDX LES MEMES DATE ET HEURE QUE TTO.HLP.
   RETOURNE true SI OK OU false SI ERREUR. *)
Var DateHeure:longint;
Begin
  (* DETERMINATION DES DATE ET HEURE DE TTO.HLP *)
  NomFichierTraite:='TTO.HLP'; assign(FichierAide,NomFichierTraite);
  if OUVERTURE_FICHIER(tf_Aide)
     then
       begin
         DateHeure:=LIRE_DATE_HEURE_FICHIER(tf_Aide);
         Void:=FERMETURE_FICHIER(tf_Aide);
         if DateHeure <> 0
            then
              begin
                (* ATTRIBUTION DES DATE ET HEURE A TTOHLP.NDX *)
                NomFichierTraite:='TTOHLP.NDX'; assign(FichierIndex,NomFichierTraite);
                if OUVERTURE_FICHIER(tf_Index)
                   then
                     begin
                       ALIGNER_DATE_HEURE_FICHIERS:=ECRIRE_DATE_HEURE_FICHIER(tf_Index,DateHeure);
                       Void:=FERMETURE_FICHIER(tf_Index);
                     end
                   else
                     ALIGNER_DATE_HEURE_FICHIERS:=false;
              end
            else
              ALIGNER_DATE_HEURE_FICHIERS:=false;
       end
     else
       ALIGNER_DATE_HEURE_FICHIERS:=false;
End;


END.
