
UNIT TTO_HARD;

(* MODULES BAS NIVEAU POUR LE PROGRAMME TTO VERSION 1.20 :
   - GESTION DE LA SOURIS;
   - RECONNAISSANCE DU HARDWARE;
   - TESTS ET GESTION DU TIMER, ECONOMISEUR D'ECRAN. *)


INTERFACE

Uses DOS,        (* GESTION DES INTERRUPTIONS *)
     TXT,        (* GESTION DE L'ECRAN EN MODE TEXTE (VERSION 2.00) *)
     TTO_DECL,   (* DECLARATIONS DE TYPES, CONSTANTES ET VARIABLES *)
     MOUSE,      (* GESTION DE LA SOURIS *)
     TESTADAP;   (* RECONNAISSANCE DU HARDWARE *)


Function TXTINITMOUSE (var Statut : integer) : boolean;
   (* INITIALISATION DU DRIVER ET DES PARAMETRES DE LA SOURIS *)
Function TXTDETECTMOUSE : boolean;
   (* DETECTE LA PRESENCE D'UN DRIVER DE SOURIS *)
Function TXTMOUSESAVEBUFFERSIZE : word;
   (* DETERMINE LA TAILLE DU BUFFER DE SAUVEGARDE DES PARAMETRES DE LA SOURIS *)
Function TXTSAVEMOUSE (Buffer : pointer) : boolean;
   (* SAUVE LES PARAMETRES DU DRIVER DE SOURIS DANS UN BUFFER *)
Function TXTLOADMOUSE (Buffer : pointer) : boolean;
   (* CHARGE DES PARAMETRES DU DRIVER DE SOURIS SAUVES PAR TXTSAVEMOUSE *)
Procedure EVENTHANDLER (Events, Buttons, x, y : word; x_Move, y_Move : integer);
   (* GESTIONNAIRE D'EVENEMENTS NORMAL STANDARD *)
Procedure EVENTHND;
   (* INTERFACE STANDARD ENTRE LE GESTIONNAIRE D'EVENEMENTS NORMAL ET LE DRIVER DE SOURIS *)
Function TXTINSTALLMOUSEEVENTHANDLER (Handler_Type : byte; Handler_Addr : pointer; Events : word) : boolean;
   (* INSTALLE UN GESTIONNAIRE D'EVENEMENTS *)
Function TXTMOUSECURSOFF : boolean;
   (* ETEINT LE CURSEUR DE LA SOURIS *)
Function TXTMOUSECURSON : boolean;
   (* ALLUME LE CURSEUR DE LA SOURIS *)
Function TXTGETMOUSECURSORCOUNTER (var Counter : word) : boolean;
   (* RETOURNE LE COMPTEUR DE CURSEUR INTERNE DU DRIVER *)
Function TXTMOUSECURSORMASKS (CharOption, NewChar, ColourOption, NewColour:byte; var And_Mask, Xor_Mask : word) : boolean;
   (* CONSTRUIT LES MASQUES AND ET XOR DU CURSEUR LOGIQUE DE LA SOURIS *)
Function TXTMOUSECURSOR (CursorType, Mask1, Mask2 : word) : boolean;
   (* DEFINIT L'ASPECT DU CURSEUR SOURIS *)

Function TXTGETCOMPUTERTYPE (var Type_Ordinateur : byte) : boolean;
   (* DETERMINE LE TYPE D'ORDINATEUR *)
Function TXTGETVIDEOSYSTEMS (var Carte1, Moniteur1, Carte2, Moniteur2 : byte) : boolean;
   (* TESTE LA (LES) CARTE(S) VIDEO ET LE (LES) MONITEUR(S) INSTALLE(S) *)

Function NUMERO_INTERRUPTION_LIBRE : boolean;
   (* CHERCHE UN N D'INTERRUPTION LIBRE *)
Procedure DETOURNEMENT_INT08;
   (* DETOURNEMENT DE L'INTERRUPTION 08H *)
Procedure RESTAURATION_INT08;
   (* RESTAURATION DE L'INT 08H DETOURNEE *)
Function VITESSE_TIMER : byte;
   (* DETERMINE LE NOMBRE DE TICS PAR SECONDE (TEST SUR 5 SECONDES) *)

Procedure ECONOMISEUR_ECRAN;
   (* "ETEINT" L'ECRAN JUSQU'A CE QU'UNE TOUCHE DU CLAVIER SOIT PRESSEE OU UNE ACTION SOURIS DETECTEE *)


IMPLEMENTATION

Function TXTINITMOUSE (var Statut : integer) : boolean;
(* INITIALISATION DU DRIVER ET DES PARAMETRES DE LA SOURIS.
   Statut CONTIENT LE CODE D'ERREUR.
   RETOURNE true SI OK OU false SI ERREUR. *)
Var Ok:boolean;
    Mot:word;
Begin
  Statut:=0;
  with MouseParams do
    begin
      Ok:=TEST_SOURIS(mp_NbButtons);
      if Ok
         then
           begin
             TYPE_SOURIS(mp_VersionHi,mp_VersionLo,mp_MouseType);
             PARAMETRES_SOURIS(mp_ButtonsDown,mp_X,mp_Y);
             mp_SaveBufferSize:=TAILLE_BUFFER_SAUVEGARDE;
             mp_Page:=PAGE_SOURIS;
             VITESSE_SOURIS(mp_XSpeed,mp_YSpeed,mp_DoubleSpeed);
             if ((mp_VersionHi * 100) + mp_VersionLo) >= 626
                then
                  begin
                    Mot:=INFORMATIONS_DRIVER;
                    if (Mot and $8000) = $0000 then mp_DriverType:=mp_Com else mp_DriverType:=mp_Sys;
                    if (Mot and $2000) = $0000
                       then
                         begin
                           if (Mot and $1000) = $0000 then mp_CursorType:=mc_Log else mp_CursorType:=mc_Electronic;
                         end
                       else
                         mp_CursorType:=mc_Graphic;
                    mp_IntFreq:=(Mot shr 8) and $000F;
                  end;
             if ((mp_VersionHi * 100) + mp_VersionLo) >= 701 then MASQUES_CURSEUR(mp_Mask1,mp_Mask2);
             if ((mp_VersionHi * 100) + mp_VersionLo) >= 702 then mp_CursorCounter:=COMPTEUR_CURSEUR;
             mp_Valid:=true;
           end
         else
           Statut:=-1;
    end;
  TXTINITMOUSE:=Ok;
End;


Function TXTDETECTMOUSE : boolean;
(* DETECTE LA PRESENCE D'UN DRIVER DE SOURIS.
   RETOURNE true SI DRIVER PRESENT OU false SI DRIVER ABSENT. *)
Var Ok:boolean;
    n:byte;   (* VARIABLE NON UTILISEE *)
Begin
  Ok:=TEST_SOURIS(n);
  TXTDETECTMOUSE:=Ok;
End;


Function TXTMOUSESAVEBUFFERSIZE : word;
(* DETERMINE LA TAILLE DU BUFFER DE SAUVEGARDE DES PARAMETRES DE LA SOURIS *)
Begin
  TXTMOUSESAVEBUFFERSIZE:=TAILLE_BUFFER_SAUVEGARDE;
End;


Function TXTSAVEMOUSE (Buffer : pointer) : boolean;
(* SAUVE LES PARAMETRES DU DRIVER DE SOURIS DANS UN BUFFER DONT LA TAILLE PEUT
   ETRE DETERMINEE PAR MouseParams.mpBufferSize OU PAR TxtMouseSaveBufferSize.
   RETOURNE true SI OK OU false SI ERREUR D'ADRESSAGE. *)
Begin
  if Buffer <> nil
     then
       begin
         TXTSAVEMOUSE:=true; SAUVEGARDER_RESTAURER_ETAT(0,Buffer);
       end
     else
       TXTSAVEMOUSE:=false;
End;


Function TXTLOADMOUSE (Buffer : pointer) : boolean;
(* CHARGE DES PARAMETRES DU DRIVER DE SOURIS SAUVES DANS UN BUFFER.
   RETOURNE true SI OK OU false SI ERREUR D'ADRESSAGE. *)
Begin
  if Buffer <> nil
     then
       begin
         TXTLOADMOUSE:=true; SAUVEGARDER_RESTAURER_ETAT(1,Buffer);
       end
     else
       TXTLOADMOUSE:=false;
End;


{$F+}
Procedure EVENTHANDLER (Events, Buttons, x, y : word; x_Move, y_Move : integer);
(* GESTIONNAIRE D'EVENEMENTS STANDARD (REPONSE A UN EVENEMENT SOURIS).
   SE CONTENTE DE CHARGER LES DONNEES DU DRIVER DANS LA VARIABLE MouseEventParams.
   - Events EST LE MASQUE DES EVENEMENTS AYANT DECLENCHE LE GESTIONNAIRE;
   - Buttons EST LE MASQUE DES BOUTONS DE LA SOURIS ENFONCES;
   - (x,y) SONT LES COORDONNEES DU CURSEUR DE LA SOURIS;
   - x_Move ET y_Move SONT LES DEPLACEMENTS DE LA SOURIS (EN MICKEYS).
   DANS LE CAS PRESENT DU PROGRAMME TTO 1.20, REMET LE COMPTEUR DE TICS DE L'ECONOMISEUR D'ECRAN A ZERO.
   me_Modified NE VAUDRA CEPENDANT true QUE SI LE BOUTON GAUCHE A ETE ACTIONNE. *)
Begin
  with MouseEventParams do
    begin
      me_Event:=Events;
      me_Modified:=not (me_Event = me_MouseMove);
      me_Buttons:=Buttons;
      if me_Modified then begin me_X:=x; me_Y:=y; end;
      me_XMove:=x_Move; me_YMove:=y_Move;
      me_Valid:=true;
    end;
  CompteurTicsEcran:=0;
End;
{$F-}


{$F+}
{$L EVENTHND}
Procedure EVENTHND; external;
(* INTERFACE ENTRE LE GESTIONNAIRE D'EVENEMENTS EVENTHANDLER ET LE DRIVER DE SOURIS *)
{$F-}


Function TXTINSTALLMOUSEEVENTHANDLER (Handler_Type : byte; Handler_Addr : pointer; Events : word) : boolean;
(* INSTALLE UN GESTIONNAIRE D'EVENEMENTS.
   Events EST LA COMBINAISON LOGIQUE (OR) DES EVENEMENTS A DETECTER.
   RETOURNE true = SI OK OU false SI ERREUR. *)
Var Ok:boolean;
Begin
  case Handler_Type of
    me_Normal:Ok:=INSTALLER_GESTIONNAIRE_EVENEMENTS(Handler_Type,Events,longint(Handler_Addr));
    me_Alternate:Ok:=INSTALLER_GESTIONNAIRE_EVENEMENTS(Handler_Type,Events,longint(Handler_Addr));
  else
    Ok:=false;
  end;
  TXTINSTALLMOUSEEVENTHANDLER:=Ok;
End;


Function TXTGETMOUSECURSORCOUNTER (var Counter : word) : boolean;
(* RETOURNE LE COMPTEUR DE CURSEUR INTERNE DU DRIVER.
   RETOURNE true SI OK OU false SI VERSION DU DRIVER INCORRECTE. *)
Var Ok:boolean;
Begin
  with MouseParams do
    begin
      if mp_Valid
         then
           begin
             if ((mp_VersionHi * 100) + mp_VersionLo) >= 702 then Ok:=true else Ok:=false;
           end
         else
           Ok:=true;
    end;
  if Ok then Counter:=COMPTEUR_CURSEUR;
  TXTGETMOUSECURSORCOUNTER:=Ok;
End;


Function TXTMOUSECURSOFF : boolean;
(* ETEINT LE CURSEUR DE LA SOURIS.
   VALEUR RENVOYEE PAR LA FONCTION : TOUJOURS true = OK. *)
Begin
  AFFICHER_EFFACER_CURSEUR(0);
  TXTMOUSECURSOFF:=true;
End;


Function TXTMOUSECURSON : boolean;
(* ALLUME LE CURSEUR DE LA SOURIS.
   VALEUR RENVOYEE PAR LA FONCTION : TOUJOURS true = OK. *)
Begin
  AFFICHER_EFFACER_CURSEUR(1);
  TXTMOUSECURSON:=true;
End;


Function TXTMOUSECURSORMASKS (CharOption, NewChar, ColourOption, NewColour : byte; var And_Mask, Xor_Mask : word) : boolean;
(* CONSTRUIT LES MASQUES AND ET XOR DU CURSEUR LOGIQUE DE LA SOURIS.
   CharOption : 0 = GARDER LE CARACTERE,
                1 = REMPLACER LE CARACTERE PAR NewChar;
   ColourOption : 0 = GARDER LA COULEUR,
                  1 = REMPLACER LA COULEUR PAR NewColour,
                  2 = INVERSER LA COULEUR.
   RETOURNE true =SI OK OU false SI OPTION INCONNUE. *)
Var Ok:boolean;
Begin
  Ok:=true;
  with Registres do
    begin
      (* MASQUE DU CARACTERE (cl ET dl) *)
      case CharOption of
        mc_Keep:begin cl:=$FF; dl:=$00; end;
        mc_Replace:begin cl:=$00; dl:=NewChar; end;
      else
        Ok:=false;
      end;
      if Ok
         then
           begin
             (* MASQUE DE LA COULEUR (ch ET dh) *)
             case ColourOption of
               mc_Keep:begin ch:=$FF; dh:=$00; end;
               mc_Replace:begin ch:=$00; dh:=NewColour; end;
               mc_Invert:begin ch:=$FF; dh:=$FF; end;
             else
               Ok:=false;
             end;
           end;
      if Ok
         then
           begin
             And_Mask:=cx; Xor_Mask:=dx;
           end;
    end;
  TXTMOUSECURSORMASKS:=Ok;
End;


Function TXTMOUSECURSOR (CursorType, Mask1, Mask2 : word) : boolean;
(* DEFINIT L'ASPECT DU CURSEUR SOURIS.
   RETOURNE true SI OK OU false SI ERREUR. *)
Begin
  TXTMOUSECURSOR:=true;
  case CursorType of
    mc_Log:DEFINITION_CURSEUR(CursorType,Mask1,Mask2);
    mc_Electronic:begin
                    if Test_Systems
                       then
                         begin
                           if TextParams.tp_ModeType = 0
                              then
                                begin   (* MODE MONOCHROME PUR *)
                                  if (Mask1 <= 15) and (Mask2 <= 15)
                                     then DEFINITION_CURSEUR(CursorType,Mask1,Mask2)
                                     else TXTMOUSECURSOR:=false;
                                end
                              else
                                begin   (* MODE COULEUR OU NIVEAUX DE GRIS *)
                                  if (Mask1 <= 7) and (Mask2 <= 7)
                                     then DEFINITION_CURSEUR(CursorType,Mask1,Mask2)
                                     else TXTMOUSECURSOR:=false;
                                end;
                         end;
                  end;
  else
    TXTMOUSECURSOR:=false;
  end;
End;


Function TXTGETCOMPUTERTYPE (var Type_Ordinateur : byte) : boolean;
(* DETERMINE LE TYPE D'ORDINATEUR.
   VALEUR RENVOYEE PAR LA FONCTION : TOUJOURS True = PAS D'ERREUR. *)
Const Inconnu = 0;
      Pc = 1;
      Pc_BiosXt = 2;
      Pc_Convertible = 3;
      PcJr = 4;
      Xt = 5;
      Xt286 = 6;
      At = 7;
      Ps2_Model_25 = 8;
      Ps2_Model_30 = 9;
      Ps2_Model_50 = 10;
      Ps2_Model_60 = 11;
      Ps2_Model_80 = 12;
      At_Ps2_Xt286 = 101;
      Ps2 = 102;
      Erreur = 255;
Type Tableau = array [0..7] of byte;
Var Donnees:^Tableau;
    Modele,Sous_Modele,Revision_Bios:byte;
Begin
  with Registres do
    begin
      ah:=$C0;
      intr($15,Registres);
      if (ah = 0) and ((flags and fcarry) = 0)
         then   (* LA FONCTION EST SUPPORTEE PAR LE BIOS *)
           begin   
             Donnees:=ptr(es,bx);   (* BUFFER EN ES:BX *)
             Modele:=Donnees^[2]; Sous_Modele:=Donnees^[3]; Revision_Bios:=Donnees^[4];
             case Modele of
               $F8:Type_Ordinateur:=Ps2_Model_80;
               $F9:Type_Ordinateur:=Pc_Convertible;
               $FA:case Sous_Modele of
                     0:Type_Ordinateur:=Ps2_Model_30;
                     1:Type_Ordinateur:=Ps2_Model_25;
                   else
                     Type_Ordinateur:=Ps2;
                   end;
               $FB:Type_Ordinateur:=Xt;
               $FC:case Sous_Modele of
                     2:Type_Ordinateur:=Xt286;
                     4:Type_Ordinateur:=Ps2_Model_50;
                     5:Type_Ordinateur:=Ps2_Model_60;
                   else
                     Type_Ordinateur:=At;
                   end;
               $FD:Type_Ordinateur:=PcJr;
               $FE:Type_Ordinateur:=Xt;
               $FF:if Revision_Bios = 2 then Type_Ordinateur:=Pc_BiosXt else Type_Ordinateur:=Pc;
             end;
           end
         else   (* LA FONCTION N'EST PAS SUPPORTEE PAR LE BIOS *)
           begin
             Modele:=mem[$F000:$FFFE];
             case Modele of
               $F8:Type_Ordinateur:=Ps2_Model_80;
               $F9:Type_Ordinateur:=Pc_Convertible;
               $FA:Type_Ordinateur:=Ps2;
               $FB:Type_Ordinateur:=Xt;
               $FC:Type_Ordinateur:=At_Ps2_Xt286;
               $FD:Type_Ordinateur:=PcJr;
               $FE:Type_Ordinateur:=Xt;
               $FF:Type_Ordinateur:=Pc;
             else
               Type_Ordinateur:=Inconnu;
             end;
           end;
    end;
  TXTGETCOMPUTERTYPE:=true;
End;


Function TXTGETVIDEOSYSTEMS (var Carte1, Moniteur1, Carte2, Moniteur2 : byte) : boolean;
(* TESTE LA (LES) CARTE(S) VIDEO ET LE (LES) MONITEUR(S) INSTALLE(S).
   RETOURNE true SI OK OU false SI ERREUR. *)
Var Ok:boolean;
    Mode_Video,s,sBis:byte;
Begin
  Ok:=TEST_VGA_MCGA(Carte1,Moniteur1,Carte2,Moniteur2);
  if Ok
     then
       begin
         if not((Carte1 = Vga) or (Carte1 = Mcga) or (Carte2 = Vga) or (Carte2 = Mcga) or
                (Carte1 = Svga_Vesa) or (Carte2 = Svga_Vesa))
            then   (* NI CARTE MCGA NI CARTE VGA *)
              begin
                Ok:=TEST_EGA(Carte1,Moniteur1);
                If Carte1 = Ega
                   then   (* TEST DE L'AUTRE SYSTEME INSTALLE *)
                     begin
                       case Moniteur1 of
                         Mono:begin
                                Ok:=TEST_CGA(Carte2,Moniteur2);
                                if Carte2 = Cga
                                   then   (* QUEL SYSTEME EST ACTIVE ? *)
                                     begin
                                       Ok:=GETMODE(Mode_Video);
                                       if (Mode_Video <> 7) and (Mode_Video <> $0F)
                                          then   (* MODE COULEUR ACTIF *)
                                            begin
                                              s:=Carte1; sBis:=Moniteur1; Carte1:=Carte2; Moniteur1:=Moniteur2;
                                              Carte2:=s; Moniteur2:=sBis;
                                            end;
                                     end
                                   else   (* PAS DE 2EME SYSTEME *)
                                     begin
                                       Carte2:=Aucun; Moniteur2:=Aucun;
                                     end;
                              end;
                         Couleur:begin
                                   Ok:=TEST_MDA_HERCULES(Carte2,Moniteur2);
                                   if (Carte2 = Mda) or (Carte2 = Hercules)
                                      then   (* QUEL SYSTEME EST ACTIVE ? *)
                                        begin
                                          Ok:=GETMODE(Mode_Video);
                                          if (Mode_Video = 7) or (Mode_Video = $0F)
                                             then   (* MODE MONO ACTIF *)
                                               begin
                                                 s:=Carte1; sBis:=Moniteur1; Carte1:=Carte2; Moniteur1:=Moniteur2;
                                                 Carte2:=s; Moniteur2:=sBis;
                                               end;
                                        end
                                      else   (* PAS DE 2EME SYSTEME *)
                                        begin
                                          Carte2:=Aucun; Moniteur2:=Aucun;
                                        end;
                                 end;
                       end;
                     end
                   else   (* PAS DE CARTE EGA *)
                     begin
                       Ok:=TEST_CGA(Carte1,Moniteur1);
                       if Carte1 = Cga
                          then   (* TEST DE L'AUTRE SYSTEME INSTALLE *)
                            begin
                              Ok:=TEST_MDA_HERCULES(Carte2,Moniteur2);
                              if (Carte2 = Mda) or (Carte2 = Hercules)
                                 then   (* QUEL SYSTEME EST ACTIVE ? *)
                                   begin
                                     Ok:=GETMODE(Mode_Video);
                                     if Mode_Video = 7
                                        then
                                          begin   (* MODE MONO ACTIF *)
                                            s:=Carte1; sBis:=Moniteur1; Carte1:=Carte2; Moniteur1:=Moniteur2;
                                            Carte2:=s; Moniteur2:=sBis;
                                          end;
                                   end
                                 else   (* PAS DE 2EME SYSTEME *)
                                   begin
                                     Carte2:=Aucun; Moniteur2:=Aucun;
                                   end;
                            end
                          else   (* PAS DE CARTE CGA *)
                            begin
                              Ok:=TEST_MDA_HERCULES(Carte1,Moniteur1);
                              Carte2:=Aucun; Moniteur2:=Aucun;   (* PAS DE 2EME SYSTEME *)
                            end;
                     end;
              end;
       end;
  TXTGETVIDEOSYSTEMS:=Ok;
End;


{$F+}
{$L DATHEURE}
Procedure DATHEURE (var Annee : word; var Mois, Jour, Heure, Minute, Seconde, Centieme : byte); external;
(* DETERMINATION DE LA DATE ET DE L'HEURE DU SYSTEME *)
{$F-}


Procedure TIC_HORLOGE_TEST; interrupt;
(* NOUVELLE INTERRUPTION 08h : APPELLE L'ANCIENNE ET INCREMENTE CompteurTicsSauvetage.
   UTILISEE UNIQUEMENT LORS DU TEST DE VITESSE DU TIMER. *)
Begin
  intr(Int08,Registres); inc(CompteurTicsSauvetage);
End;


Procedure TIC_HORLOGE; interrupt;
(* NOUVELLE INTERRUPTION 08h : APPELLE L'ANCIENNE ET INCREMENTE LES 2 COMPTEURS DE TICS.
   SI LE COMPTEUR DEPASSE LE MONTANT CORRESPONDANT AU DELAI DE SAUVEGARDE AUTOMATIQUE,
   LA VARIABLE SauverSituation EST CHARGEE AVEC true ET LE COMPTEUR EST REINITIALISE A 0.
   IDEM POUR LE DELAI DE L'ECONOMISEUR D'ECRAN. *)
Begin
  intr(Int08,Registres); inc(CompteurTicsSauvetage); inc(CompteurTicsEcran);
  if CompteurTicsSauvetage >= TicsDelaiSauvetage then SauverSituation:=true;
  if CompteurTicsEcran >= TicsDelaiEcran then EconomiserEcran:=true;
End;


Function NUMERO_INTERRUPTION_LIBRE : boolean;
(* CHERCHE UN N D'INTERRUPTION LIBRE. RENVOIE false SI PAS D'ENTREE LIBRE TROUVEE.
   SI TROUVE, CHARGE LES VALEURS DE Int08 (N NOUVELLE INTERRUPTION) ET
   AdresseInt08 (ADRESSE DE L'ANCIENNE INT 08). *)
Begin
  Int08:=255;
  repeat
    getintvec(Int08,AdresseInt08); if AdresseInt08 <> nil then dec(Int08);
  until (AdresseInt08 = nil) or (Int08 = 0);
  if Int08 <> 0 then getintvec(8,AdresseInt08);
  NUMERO_INTERRUPTION_LIBRE:=(Int08 <> 0);
End;


Procedure DETOURNEMENT_INT08_TEST;
(* DETOURNEMENT DE L'INTERRUPTION 08H.
   UTILISEE UNIQUEMENT LORS DU TEST DE VITESSE DU TIMER. *)
Begin
  setintvec(Int08,AdresseInt08); setintvec(8,@ TIC_HORLOGE_TEST);
End;


Procedure DETOURNEMENT_INT08;
(* DETOURNEMENT DE L'INTERRUPTION 08H *)
Begin
  setintvec(Int08,AdresseInt08); setintvec(8,@ TIC_HORLOGE);
End;


Procedure RESTAURATION_INT08;
(* RESTAURATION DE L'INT 08H DETOURNEE *)
Begin
  setintvec(8,AdresseInt08); setintvec(Int08,nil);
End;


Function VITESSE_TIMER : byte;
(* DETERMINE LE NOMBRE DE TICS PAR SECONDE (TEST SUR 5 SECONDES).
   RETOURNE 0 SI ERREUR OU SI TIMER TROP LENT OU TROP RAPIDE. *)
Var Annee1,Annee2:word;
    Tentatives,Mois1,Mois2,Jour1,Jour2,Heure1,Heure2,Minute1,Minute2,Seconde1,Seconde2,Centieme1,Centieme2:byte;
    NbCentiemes1,NbCentiemes2,NbTics:real;
    TicsParSeconde:longint;
    Ok:boolean;
Begin
  VITESSE_TIMER:=0;
  Tentatives:=0; Ok:=false;
  repeat
    inc(Tentatives);
    CompteurTicsSauvetage:=0;
    DATHEURE(Annee1,Mois1,Jour1,Heure1,Minute1,Seconde1,Centieme1);
    DETOURNEMENT_INT08_TEST; repeat until CompteurTicsSauvetage >= 90; RESTAURATION_INT08;
    DATHEURE(Annee2,Mois2,Jour2,Heure2,Minute2,Seconde2,Centieme2);
    if (Annee1 = Annee2) and (Mois1 = Mois2) and (Jour1 = Jour2) and (Heure1 = Heure2) and (Minute1 = Minute2)
       then   (* C'EST CALCULABLE *)
         begin
           Ok:=true;
           NbCentiemes1:=Centieme1 + (Seconde1 * 100); NbCentiemes2:=Centieme2 + (Seconde2 * 100);
           NbCentiemes2:=NbCentiemes2 - NbCentiemes1;   (* DIFFERENCE EN CENTIEMES *)
           NbTics:=CompteurTicsSauvetage * 100; TicsParSeconde:=round(NbTics / NbCentiemes2);
           if TicsParSeconde in [1..255] then VITESSE_TIMER:=TicsParSeconde;
         end;
  until (Tentatives = 3) or (Ok);
End;


Procedure ECONOMISEUR_ECRAN;
(* "ETEINT" L'ECRAN JUSQU'A CE QU'UNE TOUCHE DU CLAVIER SOIT PRESSEE OU UNE ACTION SOURIS DETECTEE.
   L'INT 09H DETOURNEE REMET CompteurTicsEcran A ZERO A CHAQUE TOUCHE PRESSEE
   (IDEM POUR CHAQUE ACTION SOURIS).
   LE COMPTEUR DE SAUVETAGE AUTOMATIQUE SUR DISQUE EST REMIS A ZERO. *)
Var x,y,x1,y1,x2,y2,Fond,Couleur:word;
    Image:videosave;
Begin
  x:=windx; y:=windy; x1:=windx1; y1:=windy1; x2:=windx2; y2:=windy2; Fond:=textbackgr; Couleur:=textcolour;
  txtunwind; txtsave(Image);
  if Souris then Void:=txtmousecursoff;
  txtback(black); txtcolour(black); txtclear;
  repeat
  until CompteurTicsEcran = 0;
  CompteurTicsSauvetage:=0; SauverSituation:=false;   (* REMISE A ZERO SAUVETAGE AUTOMATIQUE *)
  txtwind(x1,y1,x2,y2); txtload(Image); txtback(Fond); txtcolour(Couleur); txtmove(x,y);
  if Souris then Void:=txtmousecurson;
  EconomiserEcran:=false;
End;


END.
