
UNIT TTO_LIST;

(* GESTION DES LISTES CHAINEES DE POINTEURS POUR LE PROGRAMME TTO VERSION 1.20 *)

INTERFACE

Uses TTO_DECL,   (* DECLARATION DE CONSTANTES, TYPES ET VARIABLES GLOBALES *)
     TTO_COMN;   (* MESSAGES D'ERREUR *)


Procedure AJOUT_JOUEUR_LISTE_CATEGORIE (Numero, Option : byte; Pointeur : pJoueur);
   (* AJOUT DANS LA LISTE DES JOUEURS D'UNE CATEGORIE *)
Procedure SUPPRESSION_JOUEUR_LISTE_CATEGORIE (Numero : byte; Pointeur : pJoueur);
   (* SUPPRESSION DANS LA LISTE DES JOUEURS D'UNE CATEGORIE *)
Procedure AJOUT_MATCH_LISTE_CATEGORIE (Numero : byte; Pointeur : pMatch);
   (* AJOUT DANS LA LISTE DES MATCHES D'UNE CATEGORIE *)
Procedure SUPPRESSION_MATCH_LISTE_CATEGORIE (Numero : byte; Pointeur : pMatch);
   (* SUPPRESSION DANS LA LISTE DES MATCHES D'UNE CATEGORIE *)

Procedure SUPPRESSION_LISTE_JOUEURS_CATEGORIE (Numero : byte);
   (* DESALLOCATION DE LA LISTE CHAINEE DE JOUEURS D'UNE CATEGORIE *)
Procedure SUPPRESSION_LISTE_MATCHES_CATEGORIE (Numero : byte);
   (* DESALLOCATION DE LA LISTE CHAINEE DES MATCHES D'UNE CATEGORIE *)

Procedure TRI_ALPHABETIQUE_JOUEURS (Numero : byte);
   (* TRI ALPHABETIQUE DES JOUEURS D'UNE CATEGORIE *)

Function EXTRACTION_LISTE_JOUEURS (Option : byte; Poule : word; Numero : byte; var pTeteLJ, pQueueLJ : pListeJoueurs) : word;
   (* SELECTION DE JOUEURS DE LA CATEGORIE Numero SUIVANT CRITERE (RENVOIE UNE LISTE CHAINEE) *)
Procedure SUPPRESSION_LISTE_JOUEURS (pTete : pListeJoueurs);
   (* DESALLOCATION D'UNE LISTE DE JOUEURS CREEE PAR "EXTRACTION_LISTE_JOUEURS" *)
Procedure SUPPRESSION_JOUEUR_LISTE (Pointeur : pListeJoueurs; var pTete, pQueue : pListeJoueurs);
   (* SUPPRESSION D'UN JOUEUR DANS UNE LISTE CREEE PAR "EXTRACTION_LISTE_JOUEURS" *)

Procedure ANNULATION_RESULTATS_JOUEURS (Numero : byte);
   (* ANNULATION DES RESULTATS DES JOUEURS D'UNE CATEGORIE *)

Function CREATION_AUTOMATIQUE_MATCHES (Numero : byte) : word;
   (* CREATION DES MATCHES D'ELIMINATOIRES (CATEGORIE ENTIERE OU POULE PAR POULE) *)


IMPLEMENTATION

Procedure AJOUT_JOUEUR_LISTE_CATEGORIE (Numero, Option : byte; Pointeur : pJoueur);
(* AJOUT DU POINTEUR Pointeur DANS LA LISTE DES JOUEURS DE LA CATEGORIE Numero.
   OPTION : list_Alpha = INSERTION DANS L'ORDRE ALPHABETIQUE,
            list_Ajout = AJOUT EN FIN DE LISTE. *)
Var p,pAux:pJoueur;
    Trouve:boolean;
Begin
  with Categories[Numero] do
    case Option of
      list_Ajout:begin
                   if pTeteJoueurs = nil
                      then
                        begin   (* LA LISTE EST VIDE *)
                          pTeteJoueurs:=Pointeur; pQueueJoueurs:=Pointeur; Pointeur^.pPrec:=nil; Pointeur^.pSuiv:=nil;
                        end
                      else
                        begin
                          pQueueJoueurs^.pSuiv:=Pointeur; Pointeur^.pPrec:=pQueueJoueurs;
                          Pointeur^.pSuiv:=nil; pQueueJoueurs:=Pointeur;
                        end;
                 end;
      list_Alpha:begin
                   p:=pTeteJoueurs; Trouve:=false;
                   while (p <> nil) and (not Trouve) do
                     begin
                       if (Pointeur^.NomJoueur < p^.NomJoueur) or
                          ((Pointeur^.NomJoueur = p^.NomJoueur) and (Pointeur^.PrenomJoueur < p^.PrenomJoueur))
                          then   (* INSERTION DE Pointeur AVANT p *)
                            begin
                              pAux:=p^.pPrec; Pointeur^.pPrec:=pAux;
                              if pAux = nil then pTeteJoueurs:=Pointeur else pAux^.pSuiv:=Pointeur;
                              Pointeur^.pSuiv:=p; p^.pPrec:=Pointeur;
                              Trouve:=true;
                            end
                          else
                            p:=p^.pSuiv;
                     end;
                   if not Trouve
                      then   (* INSERTION EN FIN DE LISTE *)
                        begin
                          if pQueueJoueurs = nil
                             then   (* LA LISTE EST VIDE *)
                               begin
                                 pTeteJoueurs:=Pointeur; pQueueJoueurs:=Pointeur; Pointeur^.pPrec:=nil; Pointeur^.pSuiv:=nil;
                               end
                             else   (* LA LISTE N'EST PAS VIDE *)
                               begin
                                 pQueueJoueurs^.pSuiv:=Pointeur; Pointeur^.pPrec:=pQueueJoueurs;
                                 Pointeur^.pSuiv:=nil; pQueueJoueurs:=Pointeur;
                               end;
                        end;
                 end;
    end;
End;


Procedure SUPPRESSION_JOUEUR_LISTE_CATEGORIE (Numero : byte; Pointeur : pJoueur);
(* SUPPRESSION DU POINTEUR Pointeur DANS LA LISTE DES JOUEURS DE LA CATEGORIE Numero *)
Var p:pJoueur;
Begin
  with Categories[Numero] do
    begin
      if Pointeur^.pPrec = nil
         then   (* C'EST LE 1ER JOUEUR DE LA LISTE *)
           begin
             p:=Pointeur^.pSuiv;
             if p = nil
                then
                  begin   (* EGALEMENT LE DERNIER DE LA LISTE *)
                    pTeteJoueurs:=nil; pQueueJoueurs:=nil;
                  end
                else
                  begin   (* CE N'EST PAS LE DERNIER DE LA LISTE *)
                    pTeteJoueurs:=p; p^.pPrec:=nil;
                  end;
           end
         else   (* CE N'EST PAS LE 1ER JOUEUR DE LA LISTE *)
           begin
             p:=Pointeur^.pSuiv;
             if p = nil
                then   (* MAIS C'EST LE DERNIER DE LA LISTE *)
                  begin
                    p:=Pointeur^.pPrec; pQueueJoueurs:=p; p^.pSuiv:=nil;
                  end
                else   (* CE N'EST PAS NON PLUS LE DERNIER DE LA LISTE *)
                  begin
                    p^.pPrec:=Pointeur^.pPrec; p:=Pointeur^.pPrec; p^.pSuiv:=Pointeur^.pSuiv;
                  end;
           end;
      if Pointeur <> nil then dispose(Pointeur);
    end;
End;


Procedure AJOUT_MATCH_LISTE_CATEGORIE (Numero : byte; Pointeur : pMatch);
(* AJOUT DU POINTEUR Pointeur DANS LA LISTE DES MATCHES DE LA CATEGORIE Numero *)
Begin
  with Categories[Numero] do
  if pTeteMatches = nil
     then
       begin   (* LA LISTE EST VIDE *)
         pTeteMatches:=Pointeur; pQueueMatches:=Pointeur; Pointeur^.pPrec:=nil; Pointeur^.pSuiv:=nil;
       end
     else   (* AJOUT EN FIN DE LISTE *)
       begin
         pQueueMatches^.pSuiv:=Pointeur; Pointeur^.pPrec:=pQueueMatches;
         Pointeur^.pSuiv:=nil; pQueueMatches:=Pointeur;
       end;
End;


Procedure SUPPRESSION_MATCH_LISTE_CATEGORIE (Numero : byte; Pointeur : pMatch);
(* SUPPRESSION DU POINTEUR Pointeur DANS LA LISTE DES MATCHES DE LA CATEGORIE Numero *)
Var p:pMatch;
Begin
  with Categories[Numero] do
    begin
      if Pointeur^.pPrec = nil
         then   (* C'EST LE 1ER MATCH DE LA LISTE *)
           begin
             p:=Pointeur^.pSuiv;
             if p = nil
                then
                  begin   (* EGALEMENT LE DERNIER DE LA LISTE *)
                    pTeteMatches:=nil; pQueueMatches:=nil;
                  end
                else
                  begin   (* CE N'EST PAS LE DERNIER DE LA LISTE *)
                    pTeteMatches:=p; p^.pPrec:=nil;
                  end;
           end
         else   (* CE N'EST PAS LE 1ER MATCH DE LA LISTE *)
           begin
             p:=Pointeur^.pSuiv;
             if p = nil
                then   (* MAIS C'EST LE DERNIER DE LA LISTE *)
                  begin
                    p:=Pointeur^.pPrec; pQueueMatches:=p; p^.pSuiv:=nil;
                  end
                else   (* CE N'EST PAS NON PLUS LE DERNIER DE LA LISTE *)
                  begin
                    p^.pPrec:=Pointeur^.pPrec; p:=Pointeur^.pPrec; p^.pSuiv:=Pointeur^.pSuiv;
                  end;
           end;
      if Pointeur <> nil then dispose(Pointeur);
    end;
End;


Procedure SUPPRESSION_LISTE_JOUEURS_CATEGORIE (Numero : byte);
(* DESALLOCATION DE LA LISTE CHAINEE DE JOUEURS DE LA CATEGORIE Numero *)
Var p1,p2:pJoueur;
Begin
  with Categories[Numero] do
    begin
      p1:=pTeteJoueurs; while p1 <> nil do begin p2:=p1^.pSuiv; dispose(p1); p1:=p2; end;
      pTeteJoueurs:=nil; pQueueJoueurs:=nil;
    end;
End;


Procedure SUPPRESSION_LISTE_MATCHES_CATEGORIE (Numero : byte);
(* LIBERATION DE LA LISTE CHAINEE DE MATCHES DE LA CATEGORIE Numero *)
Var p1,p2:pMatch;
Begin
  with Categories[Numero] do
    begin
      p1:=pTeteMatches; while p1 <> nil do begin p2:=p1^.pSuiv; dispose(p1); p1:=p2; end;
      pTeteMatches:=nil; pQueueMatches:=nil;
    end;
End;


Procedure TRI_ALPHABETIQUE_JOUEURS (Numero : byte);
(* TRI ALPHABETIQUE DES JOUEURS DE LA CATEGORIE Numero *)
Var pCourant,p,pTrouve:pJoueur;
Begin
  with Categories[Numero] do
    begin
      pCourant:=pTeteJoueurs;
      while pCourant <> nil do
        begin
          pTrouve:=pCourant; p:=pCourant^.pSuiv;
          while p <> nil do
            begin
              if (p^.NomJoueur < pTrouve^.NomJoueur) or
                 ((p^.NomJoueur = pTrouve^.NomJoueur) and (p^.PrenomJoueur < pTrouve^.PrenomJoueur))
                 then
                   pTrouve:=p;
              p:=p^.pSuiv;
            end;
          (* pTrouve POINTE SUR LE PLUS PETIT NOM/PRENOM DANS LE RESTE DE LA LISTE APRES pCourant *)
          if pTrouve <> pCourant
             then   (* DEPLACEMENT DE pTrouve AVANT pCourant *)
               begin
                 (* SUPPRESSION DE pTrouve *)
                 if pTrouve^.pSuiv = nil
                    then   (* pTrouve EST LA QUEUE DE LISTE *)
                      begin
                        pQueueJoueurs:=pTrouve^.pPrec; pQueueJoueurs^.pSuiv:=nil;
                      end
                    else   (* pTrouve EST AU BEAU MILIEU DE LA LISTE *)
                      begin
                        p:=pTrouve^.pPrec; p^.pSuiv:=pTrouve^.pSuiv; p:=pTrouve^.pSuiv; p^.pPrec:=pTrouve^.pPrec;
                      end;
                 (* INSERTION DE pTrouve AVANT pCourant *)
                 if pCourant^.pPrec = nil
                    then   (* pCourant EST LA TETE DE LISTE *)
                      begin
                        pCourant^.pPrec:=pTrouve; pTrouve^.pSuiv:=pCourant; pTrouve^.pPrec:=nil; pTeteJoueurs:=pTrouve;
                      end
                    else   (* pCourant EST AU BEAU MILIEU DE LA LISTE *)
                      begin
                        p:=pCourant^.pPrec;
                        p^.pSuiv:=pTrouve; pTrouve^.pPrec:=p; pTrouve^.pSuiv:=pCourant; pCourant^.pPrec:=pTrouve;
                      end;
               end
             else
               pCourant:=pCourant^.pSuiv;
        end;
    end;
End;


Procedure TRI_LISTE_JOUEURS (Option : byte; var pTeteLJ, pQueueLJ : pListeJoueurs);
(* TRI D'UNE LISTE DE JOUEURS EXTRAITE PAR "EXTRACTION_LISTE_JOUEURS".
   Option : list_Alpha = PAR ORDRE ALPHABETIQUE,
            list_Poule = PAR N CROISSANT DE POULE,
            list_Clas = PAR ORDRE DECROISSANT DES RESULTATS. *)
Var pCourant,p,pTrouve:pListeJoueurs;
    Nom1,Nom2,Prenom1,Prenom2:string[15];
    Poule1,Poule2:word;
    Points1,Sets1,Score1,Points2,Sets2,Score2:integer;
Begin
  pCourant:=pTeteLJ;
  while pCourant <> nil do
    begin
      pTrouve:=pCourant; p:=pCourant^.pSuiv;
      while p <> nil do
        begin
          case Option of
            list_Alpha:begin
                         with p^ do with PointeurJoueur^ do begin Nom1:=NomJoueur; Prenom1:=PrenomJoueur; end;
                         with pTrouve^ do with PointeurJoueur^ do begin Nom2:=NomJoueur; Prenom2:=PrenomJoueur; end;
                         if (Nom1 < Nom2) or ((Nom1 = Nom2) and (Prenom1 < Prenom2)) then pTrouve:=p;
                       end;
            list_Poule:begin
                         with p^ do Poule1:=PointeurJoueur^.Poule;
                         with pTrouve^ do Poule2:=PointeurJoueur^.Poule;
                         if Poule1 < Poule2 then pTrouve:=p;
                       end;
            list_Clas:begin
                        with p^ do with PointeurJoueur^ do begin Points1:=Points; Sets1:=Sets; Score1:=Score; end;
                        with pTrouve^ do with PointeurJoueur^ do begin Points2:=Points; Sets2:=Sets; Score2:=Score; end;
                        if (Points1 > Points2) or
                           ((Points1 = Points2) and ((Sets1 > Sets2) or ((Sets1 = Sets2) and (Score1 > Score2))))
                           then pTrouve:=p;
                      end;
          end;
          p:=p^.pSuiv;
        end;
      (* pTrouve POINTE SUR LE PLUS PETIT NOM/PRENOM DANS LE RESTE DE LA LISTE APRES pCourant *)
      if pTrouve <> pCourant
         then   (* DEPLACEMENT DE pTrouve AVANT pCourant *)
           begin
             (* SUPPRESSION DE pTrouve *)
             if pTrouve^.pSuiv = nil
                then   (* pTrouve EST LA QUEUE DE LISTE *)
                  begin
                    pQueueLJ:=pTrouve^.pPrec; pQueueLJ^.pSuiv:=nil;
                  end
                else   (* pTrouve EST AU BEAU MILIEU DE LA LISTE *)
                  begin
                    p:=pTrouve^.pPrec; p^.pSuiv:=pTrouve^.pSuiv; p:=pTrouve^.pSuiv; p^.pPrec:=pTrouve^.pPrec;
                  end;
             (* INSERTION DE pTrouve AVANT pCourant *)
             if pCourant^.pPrec = nil
                then   (* pCourant EST LA TETE DE LISTE *)
                  begin
                    pCourant^.pPrec:=pTrouve; pTrouve^.pSuiv:=pCourant; pTrouve^.pPrec:=nil; pTeteLJ:=pTrouve;
                  end
                else   (* pCourant EST AU BEAU MILIEU DE LA LISTE *)
                  begin
                    p:=pCourant^.pPrec;
                    p^.pSuiv:=pTrouve; pTrouve^.pPrec:=p; pTrouve^.pSuiv:=pCourant; pCourant^.pPrec:=pTrouve;
                  end;
           end
         else
           pCourant:=pCourant^.pSuiv;
    end;
End;


Function EXTRACTION_LISTE_JOUEURS (Option : byte; Poule : word; Numero : byte; var pTeteLJ, pQueueLJ : pListeJoueurs) : word;
(* SELECTION DE JOUEURS DE LA CATEGORIE Numero SUIVANT CRITERE (RENVOIE UNE LISTE CHAINEE).
   Option : list_PouleNormal = APPARTENANT A LA POULE Poule, SANS CLASSEMENT;
            list_PouleAlpha = APPARTENANT A LA POULE Poule, PAR ORDRE ALPHABETIQUE;
            list_PouleClas = APPARTENANT A LA POULE Poule, CLASSEMENT SELON RESULTATS.
            list_LiceNormal = CATEGORIE ENTIERE, JOUEURS EN LICE, SANS CLASSEMENT;
            list_LiceAlpha = CATEGORIE ENTIERE, JOUEURS EN LICE, PAR ORDRE ALPHABETIQUE;
            list_LiceClas = CATEGORIE ENTIERE, JOUEURS EN LICE, CLASSEMENT SELON RESULTATS;
            list_LiceClasPoule = CATEGORIE ENTIERE, JOUEURS EN LICE, POULE PAR POULE, CLASSEMENT SELON RESULTATS.
   RETOURNE LE NOMBRE DE JOUEURS EXTRAITS (0 SI ERREUR D'ALLOCATION MEMOIRE). *)
Var pJ,pJAux:pJoueur;
    pL,pLAux,pLBis:pListeJoueurs;
    NbJoueurs:word;
    Ok,Trouve:boolean;
Begin
  pTeteLJ:=nil; pQueueLJ:=nil; NbJoueurs:=0;
  if Option in [list_PouleNormal,list_PouleAlpha,list_PouleClas]
     then
       begin
         (* EXTRACTION DE LA LISTE *)
         pJ:=Categories[Numero].pTeteJoueurs; Ok:=false;
         while (pJ <> nil) and (not Ok) do
           begin
             if pJ^.Poule = Poule
                then
                  begin
                    new(pL);
                    if pL <> nil
                       then
                         begin
                           inc(NbJoueurs); pL^.PointeurJoueur:=pJ; pL^.Selection:=false;
                           if pTeteLJ = nil
                              then   (* LA LISTE EST VIDE *)
                                begin
                                  pTeteLJ:=pL; pQueueLJ:=pL; pL^.pPrec:=nil; pL^.pSuiv:=nil;
                                end
                              else   (* LA LISTE N'EST PAS VIDE : INSERTION EN FIN DE LISTE *)
                                begin
                                  pQueueLJ^.pSuiv:=pL; pL^.pPrec:=pQueueLJ; pQueueLJ:=pL; pL^.pSuiv:=nil;
                                end;
                         end
                       else
                         begin
                           Void:=MESSAGE_ERREUR(1001); Ok:=true; NbJoueurs:=0;
                         end;
                  end;
             pJ:=pJ^.pSuiv;
           end;
         if NbJoueurs > 0
            then   (* TRI EVENTUEL DE LA LISTE *)
              case Option of
                list_PouleAlpha:TRI_LISTE_JOUEURS(list_Alpha,pTeteLJ,pQueueLJ);
                list_PouleClas:TRI_LISTE_JOUEURS(list_Clas,pTeteLJ,pQueueLJ);
              end;
       end
     else
       begin
         (* EXTRACTION DE LA LISTE *)
         pJ:=Categories[Numero].pTeteJoueurs; Ok:=false;
         while (pJ <> nil) and (not Ok) do
           begin
             if pJ^.EnLice
                then
                  begin
                    new(pL);
                    if pL <> nil
                       then
                         begin
                           inc(NbJoueurs); pL^.PointeurJoueur:=pJ; pL^.Selection:=false;
                           if pTeteLJ = nil
                              then   (* LA LISTE EST VIDE *)
                                begin
                                  pTeteLJ:=pL; pQueueLJ:=pL; pL^.pPrec:=nil; pL^.pSuiv:=nil;
                                end
                              else   (* LA LISTE N'EST PAS VIDE : INSERTION EN FIN DE LISTE *)
                                begin
                                  pQueueLJ^.pSuiv:=pL; pL^.pPrec:=pQueueLJ; pQueueLJ:=pL; pL^.pSuiv:=nil;
                                end;
                         end
                       else
                         begin
                           Void:=MESSAGE_ERREUR(1001); Ok:=true; NbJoueurs:=0;
                         end;
                  end;
             pJ:=pJ^.pSuiv;
           end;
         if NbJoueurs > 0
            then   (* TRI EVENTUEL DE LA LISTE *)
              Begin
                case Option of
                  list_LiceAlpha:TRI_LISTE_JOUEURS(list_Alpha,pTeteLJ,pQueueLJ);
                  list_LiceClas:TRI_LISTE_JOUEURS(list_Clas,pTeteLJ,pQueueLJ);
                  list_LiceClasPoule:begin
                                       TRI_LISTE_JOUEURS(list_Clas,pTeteLJ,pQueueLJ);
                                       TRI_LISTE_JOUEURS(list_Poule,pTeteLJ,pQueueLJ);
                                     end;
                end;
              End;
       end;
  EXTRACTION_LISTE_JOUEURS:=NbJoueurs;
End;


Procedure SUPPRESSION_LISTE_JOUEURS (pTete : pListeJoueurs);
(* DESALLOCATION DE LA LISTE CHAINEE DE JOUEURS CREEE PAR "EXTRACTION_LISTE_JOUEURS" POINTEE PAR pTete *)
Var p1,p2:pListeJoueurs;
Begin
  p1:=pTete; while p1 <> nil do begin p2:=p1^.pSuiv; dispose(p1); p1:=p2; end;
End;


Procedure SUPPRESSION_JOUEUR_LISTE (Pointeur : pListeJoueurs; var pTete, pQueue : pListeJoueurs);
(* SUPPRESSION DU POINTEUR Pointeur DANS UNE LISTE DE JOUEURS CREEE PAR "EXTRACTION_LISTE_JOUEURS" *)
Var p:pListeJoueurs;
Begin
  if Pointeur^.pPrec = nil
     then   (* C'EST LE 1ER JOUEUR DE LA LISTE *)
       begin
         p:=Pointeur^.pSuiv;
         if p = nil
            then
              begin   (* EGALEMENT LE DERNIER DE LA LISTE *)
                pTete:=nil; pQueue:=nil;
              end
            else
              begin   (* CE N'EST PAS LE DERNIER DE LA LISTE *)
                pTete:=p; p^.pPrec:=nil;
              end;
       end
     else   (* CE N'EST PAS LE 1ER JOUEUR DE LA LISTE *)
       begin
         p:=Pointeur^.pSuiv;
         if p = nil
            then   (* MAIS C'EST LE DERNIER DE LA LISTE *)
              begin
                p:=Pointeur^.pPrec; pQueue:=p; p^.pSuiv:=nil;
              end
            else   (* CE N'EST PAS NON PLUS LE DERNIER DE LA LISTE *)
              begin
                p^.pPrec:=Pointeur^.pPrec; p:=Pointeur^.pPrec; p^.pSuiv:=Pointeur^.pSuiv;
              end;
       end;
  if Pointeur <> nil then dispose(Pointeur);
End;


Procedure ANNULATION_RESULTATS_JOUEURS (Numero : byte);
(* ANNULATION DES RESULTATS DES JOUEURS DE LA CATEGORIE Numero *)
Var p:pJoueur;
Begin
  p:=Categories[Numero].pTeteJoueurs;
  while p <> nil do
    begin
      with p^ do begin MatchesJoues:=0; Points:=0; Sets:=0; Score:=0; end;
      p:=p^.pSuiv;
    end;
End;


Function CREATION_AUTOMATIQUE_MATCHES (Numero : byte) : word;
(* CREATION DES MATCHES D'ELIMINATOIRES (CATEGORIE ENTIERE OU POULE PAR POULE).
   RETOURNE LE NOMBRE DE MATCHES CREES. *)
Var pTeteLJ,pQueueLJ,pLJ1,pLJ2:pListeJoueurs;
    pM:pMatch;
    NbJoueurs,NbMatches,Poule:word;
    i,j:byte;
    Stop:boolean;
Begin
  Categories[Numero].pTeteMatches:=nil; Categories[Numero].pQueueMatches:=nil; NbMatches:=0;
  if Categories[Numero].NbPoules = 0
     then   (* LA CATEGORIE N'EST PAS DIVISEE EN POULES *)
       begin
         NbJoueurs:=EXTRACTION_LISTE_JOUEURS(list_LiceNormal,0,Numero,pTeteLJ,pQueueLJ);
         if NbJoueurs > 0
            then
              begin
                pLJ1:=pTeteLJ; Stop:=false;
                while (pLJ1 <> nil) and (not Stop) do
                  begin
                    pLJ2:=pLJ1^.pSuiv;
                    while (pLJ2 <> nil) and (not Stop) do
                      begin
                        new(pM);
                        if pM <> nil
                           then
                             begin
                               inc(NbMatches);
                               pM^.pJoueur1:=pLJ1^.PointeurJoueur; pM^.pJoueur2:=pLJ2^.PointeurJoueur; pM^.Poule:=0;
                               for i:=1 to 5 do for j:=1 to 2 do pM^.Resultats[i,j]:=0;
                               AJOUT_MATCH_LISTE_CATEGORIE(Numero,pM);
                             end
                           else
                             begin
                               Void:=MESSAGE_ERREUR(1001); Stop:=true;
                             end;
                        pLJ2:=pLJ2^.pSuiv;
                      end;
                    pLJ1:=pLJ1^.pSuiv;
                  end;
              end;
         if pTeteLJ <> nil then SUPPRESSION_LISTE_JOUEURS(pTeteLJ);
       end
     else   (* LA CATEGORIE EST DIVISEE EN POULES *)
       begin
         Poule:=1; Stop:=false;
         while (Poule <= Categories[Numero].NbPoules) and (not Stop) do
           begin
             NbJoueurs:=EXTRACTION_LISTE_JOUEURS(list_PouleNormal,Poule,Numero,pTeteLJ,pQueueLJ);
             if NbJoueurs > 0
                then
                  begin
                    pLJ1:=pTeteLJ; Stop:=false;
                    while (pLJ1 <> nil) and (not Stop) do
                      begin
                        pLJ2:=pLJ1^.pSuiv;
                        while (pLJ2 <> nil) and (not Stop) do
                          begin
                            new(pM);
                            if pM <> nil
                               then
                                 begin
                                   inc(NbMatches);
                                   pM^.pJoueur1:=pLJ1^.PointeurJoueur; pM^.pJoueur2:=pLJ2^.PointeurJoueur; pM^.Poule:=Poule;
                                   for i:=1 to 5 do for j:=1 to 2 do pM^.Resultats[i,j]:=0;
                                   AJOUT_MATCH_LISTE_CATEGORIE(Numero,pM);
                                 end
                               else
                                 begin
                                   Void:=MESSAGE_ERREUR(1001); Stop:=true;
                                 end;
                            pLJ2:=pLJ2^.pSuiv;
                          end;
                        pLJ1:=pLJ1^.pSuiv;
                      end;
                    if pTeteLJ <> nil then SUPPRESSION_LISTE_JOUEURS(pTeteLJ);
                    inc(Poule);
                  end;
           end;
       end;
  if Stop
     then
       begin
         if Categories[Numero].pTeteMatches <> nil then SUPPRESSION_LISTE_MATCHES_CATEGORIE(Numero);
         NbMatches:=0; Categories[Numero].pTeteMatches:=nil; Categories[Numero].pQueueMatches:=nil;
       end;
  CREATION_AUTOMATIQUE_MATCHES:=NbMatches;
End;


END.
